# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/base/v1beta1/coin.proto
# plugin: python-betterproto
from dataclasses import dataclass

import betterproto


@dataclass(eq=False, repr=False)
class Coin(betterproto.Message):
    """
    Coin defines a token with a denomination and an amount. NOTE: The amount
    field is an Int which implements the custom method signatures required by
    gogoproto.
    """

    denom: str = betterproto.string_field(1)
    amount: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class DecCoin(betterproto.Message):
    """
    DecCoin defines a token with a denomination and a decimal amount. NOTE: The
    amount field is an Dec which implements the custom method signatures
    required by gogoproto.
    """

    denom: str = betterproto.string_field(1)
    amount: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class IntProto(betterproto.Message):
    """IntProto defines a Protobuf wrapper around an Int object."""

    int: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class DecProto(betterproto.Message):
    """DecProto defines a Protobuf wrapper around a Dec object."""

    dec: str = betterproto.string_field(1)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/base/reflection/v2alpha1/reflection.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class AppDescriptor(betterproto.Message):
    """AppDescriptor describes a cosmos-sdk based application"""

    authn: "AuthnDescriptor" = betterproto.message_field(1)
    """
    AuthnDescriptor provides information on how to authenticate transactions on
    the application NOTE: experimental and subject to change in future
    releases.
    """

    chain: "ChainDescriptor" = betterproto.message_field(2)
    """chain provides the chain descriptor"""

    codec: "CodecDescriptor" = betterproto.message_field(3)
    """codec provides metadata information regarding codec related types"""

    configuration: "ConfigurationDescriptor" = betterproto.message_field(4)
    """
    configuration provides metadata information regarding the sdk.Config type
    """

    query_services: "QueryServicesDescriptor" = betterproto.message_field(5)
    """
    query_services provides metadata information regarding the available
    queriable endpoints
    """

    tx: "TxDescriptor" = betterproto.message_field(6)
    """
    tx provides metadata information regarding how to send transactions to the
    given application
    """


@dataclass(eq=False, repr=False)
class TxDescriptor(betterproto.Message):
    """TxDescriptor describes the accepted transaction type"""

    fullname: str = betterproto.string_field(1)
    """
    fullname is the protobuf fullname of the raw transaction type (for instance
    the tx.Tx type) it is not meant to support polymorphism of transaction
    types, it is supposed to be used by reflection clients to understand if
    they can handle a specific transaction type in an application.
    """

    msgs: List["MsgDescriptor"] = betterproto.message_field(2)
    """msgs lists the accepted application messages (sdk.Msg)"""


@dataclass(eq=False, repr=False)
class AuthnDescriptor(betterproto.Message):
    """
    AuthnDescriptor provides information on how to sign transactions without
    relying on the online RPCs GetTxMetadata and CombineUnsignedTxAndSignatures
    """

    sign_modes: List["SigningModeDescriptor"] = betterproto.message_field(1)
    """sign_modes defines the supported signature algorithm"""


@dataclass(eq=False, repr=False)
class SigningModeDescriptor(betterproto.Message):
    """
    SigningModeDescriptor provides information on a signing flow of the
    application NOTE(fdymylja): here we could go as far as providing an entire
    flow on how to sign a message given a SigningModeDescriptor, but it's
    better to think about this another time
    """

    name: str = betterproto.string_field(1)
    """name defines the unique name of the signing mode"""

    number: int = betterproto.int32_field(2)
    """number is the unique int32 identifier for the sign_mode enum"""

    authn_info_provider_method_fullname: str = betterproto.string_field(3)
    """
    authn_info_provider_method_fullname defines the fullname of the method to
    call to get the metadata required to authenticate using the provided
    sign_modes
    """


@dataclass(eq=False, repr=False)
class ChainDescriptor(betterproto.Message):
    """ChainDescriptor describes chain information of the application"""

    id: str = betterproto.string_field(1)
    """id is the chain id"""


@dataclass(eq=False, repr=False)
class CodecDescriptor(betterproto.Message):
    """
    CodecDescriptor describes the registered interfaces and provides metadata
    information on the types
    """

    interfaces: List["InterfaceDescriptor"] = betterproto.message_field(1)
    """interfaces is a list of the registerted interfaces descriptors"""


@dataclass(eq=False, repr=False)
class InterfaceDescriptor(betterproto.Message):
    """InterfaceDescriptor describes the implementation of an interface"""

    fullname: str = betterproto.string_field(1)
    """fullname is the name of the interface"""

    interface_accepting_messages: List["InterfaceAcceptingMessageDescriptor"] = betterproto.message_field(2)
    """
    interface_accepting_messages contains information regarding the proto
    messages which contain the interface as google.protobuf.Any field
    """

    interface_implementers: List["InterfaceImplementerDescriptor"] = betterproto.message_field(3)
    """
    interface_implementers is a list of the descriptors of the interface
    implementers
    """


@dataclass(eq=False, repr=False)
class InterfaceImplementerDescriptor(betterproto.Message):
    """InterfaceImplementerDescriptor describes an interface implementer"""

    fullname: str = betterproto.string_field(1)
    """fullname is the protobuf queryable name of the interface implementer"""

    type_url: str = betterproto.string_field(2)
    """
    type_url defines the type URL used when marshalling the type as any this is
    required so we can provide type safe google.protobuf.Any marshalling and
    unmarshalling, making sure that we don't accept just 'any' type in our
    interface fields
    """


@dataclass(eq=False, repr=False)
class InterfaceAcceptingMessageDescriptor(betterproto.Message):
    """
    InterfaceAcceptingMessageDescriptor describes a protobuf message which
    contains an interface represented as a google.protobuf.Any
    """

    fullname: str = betterproto.string_field(1)
    """
    fullname is the protobuf fullname of the type containing the interface
    """

    field_descriptor_names: List[str] = betterproto.string_field(2)
    """
    field_descriptor_names is a list of the protobuf name (not fullname) of the
    field which contains the interface as google.protobuf.Any (the interface is
    the same, but it can be in multiple fields of the same proto message)
    """


@dataclass(eq=False, repr=False)
class ConfigurationDescriptor(betterproto.Message):
    """
    ConfigurationDescriptor contains metadata information on the sdk.Config
    """

    bech32_account_address_prefix: str = betterproto.string_field(1)
    """bech32_account_address_prefix is the account address prefix"""


@dataclass(eq=False, repr=False)
class MsgDescriptor(betterproto.Message):
    """
    MsgDescriptor describes a cosmos-sdk message that can be delivered with a
    transaction
    """

    msg_type_url: str = betterproto.string_field(1)
    """msg_type_url contains the TypeURL of a sdk.Msg."""


@dataclass(eq=False, repr=False)
class GetAuthnDescriptorRequest(betterproto.Message):
    """
    GetAuthnDescriptorRequest is the request used for the GetAuthnDescriptor
    RPC
    """

    pass


@dataclass(eq=False, repr=False)
class GetAuthnDescriptorResponse(betterproto.Message):
    """
    GetAuthnDescriptorResponse is the response returned by the
    GetAuthnDescriptor RPC
    """

    authn: "AuthnDescriptor" = betterproto.message_field(1)
    """
    authn describes how to authenticate to the application when sending
    transactions
    """


@dataclass(eq=False, repr=False)
class GetChainDescriptorRequest(betterproto.Message):
    """
    GetChainDescriptorRequest is the request used for the GetChainDescriptor
    RPC
    """

    pass


@dataclass(eq=False, repr=False)
class GetChainDescriptorResponse(betterproto.Message):
    """
    GetChainDescriptorResponse is the response returned by the
    GetChainDescriptor RPC
    """

    chain: "ChainDescriptor" = betterproto.message_field(1)
    """chain describes application chain information"""


@dataclass(eq=False, repr=False)
class GetCodecDescriptorRequest(betterproto.Message):
    """
    GetCodecDescriptorRequest is the request used for the GetCodecDescriptor
    RPC
    """

    pass


@dataclass(eq=False, repr=False)
class GetCodecDescriptorResponse(betterproto.Message):
    """
    GetCodecDescriptorResponse is the response returned by the
    GetCodecDescriptor RPC
    """

    codec: "CodecDescriptor" = betterproto.message_field(1)
    """
    codec describes the application codec such as registered interfaces and
    implementations
    """


@dataclass(eq=False, repr=False)
class GetConfigurationDescriptorRequest(betterproto.Message):
    """
    GetConfigurationDescriptorRequest is the request used for the
    GetConfigurationDescriptor RPC
    """

    pass


@dataclass(eq=False, repr=False)
class GetConfigurationDescriptorResponse(betterproto.Message):
    """
    GetConfigurationDescriptorResponse is the response returned by the
    GetConfigurationDescriptor RPC
    """

    config: "ConfigurationDescriptor" = betterproto.message_field(1)
    """config describes the application's sdk.Config"""


@dataclass(eq=False, repr=False)
class GetQueryServicesDescriptorRequest(betterproto.Message):
    """
    GetQueryServicesDescriptorRequest is the request used for the
    GetQueryServicesDescriptor RPC
    """

    pass


@dataclass(eq=False, repr=False)
class GetQueryServicesDescriptorResponse(betterproto.Message):
    """
    GetQueryServicesDescriptorResponse is the response returned by the
    GetQueryServicesDescriptor RPC
    """

    queries: "QueryServicesDescriptor" = betterproto.message_field(1)
    """queries provides information on the available queryable services"""


@dataclass(eq=False, repr=False)
class GetTxDescriptorRequest(betterproto.Message):
    """
    GetTxDescriptorRequest is the request used for the GetTxDescriptor RPC
    """

    pass


@dataclass(eq=False, repr=False)
class GetTxDescriptorResponse(betterproto.Message):
    """
    GetTxDescriptorResponse is the response returned by the GetTxDescriptor RPC
    """

    tx: "TxDescriptor" = betterproto.message_field(1)
    """
    tx provides information on msgs that can be forwarded to the application
    alongside the accepted transaction protobuf type
    """


@dataclass(eq=False, repr=False)
class QueryServicesDescriptor(betterproto.Message):
    """
    QueryServicesDescriptor contains the list of cosmos-sdk queriable services
    """

    query_services: List["QueryServiceDescriptor"] = betterproto.message_field(1)
    """query_services is a list of cosmos-sdk QueryServiceDescriptor"""


@dataclass(eq=False, repr=False)
class QueryServiceDescriptor(betterproto.Message):
    """QueryServiceDescriptor describes a cosmos-sdk queryable service"""

    fullname: str = betterproto.string_field(1)
    """fullname is the protobuf fullname of the service descriptor"""

    is_module: bool = betterproto.bool_field(2)
    """
    is_module describes if this service is actually exposed by an application's
    module
    """

    methods: List["QueryMethodDescriptor"] = betterproto.message_field(3)
    """methods provides a list of query service methods"""


@dataclass(eq=False, repr=False)
class QueryMethodDescriptor(betterproto.Message):
    """
    QueryMethodDescriptor describes a queryable method of a query service no
    other info is provided beside method name and tendermint queryable path
    because it would be redundant with the grpc reflection service
    """

    name: str = betterproto.string_field(1)
    """name is the protobuf name (not fullname) of the method"""

    full_query_path: str = betterproto.string_field(2)
    """
    full_query_path is the path that can be used to query this method via
    tendermint abci.Query
    """


class ReflectionServiceStub(betterproto.ServiceStub):
    async def get_authn_descriptor(
        self,
        get_authn_descriptor_request: "GetAuthnDescriptorRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetAuthnDescriptorResponse":
        return await self._unary_unary(
            "/cosmos.base.reflection.v2alpha1.ReflectionService/GetAuthnDescriptor",
            get_authn_descriptor_request,
            GetAuthnDescriptorResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_chain_descriptor(
        self,
        get_chain_descriptor_request: "GetChainDescriptorRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetChainDescriptorResponse":
        return await self._unary_unary(
            "/cosmos.base.reflection.v2alpha1.ReflectionService/GetChainDescriptor",
            get_chain_descriptor_request,
            GetChainDescriptorResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_codec_descriptor(
        self,
        get_codec_descriptor_request: "GetCodecDescriptorRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetCodecDescriptorResponse":
        return await self._unary_unary(
            "/cosmos.base.reflection.v2alpha1.ReflectionService/GetCodecDescriptor",
            get_codec_descriptor_request,
            GetCodecDescriptorResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_configuration_descriptor(
        self,
        get_configuration_descriptor_request: "GetConfigurationDescriptorRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetConfigurationDescriptorResponse":
        return await self._unary_unary(
            "/cosmos.base.reflection.v2alpha1.ReflectionService/GetConfigurationDescriptor",
            get_configuration_descriptor_request,
            GetConfigurationDescriptorResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_query_services_descriptor(
        self,
        get_query_services_descriptor_request: "GetQueryServicesDescriptorRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetQueryServicesDescriptorResponse":
        return await self._unary_unary(
            "/cosmos.base.reflection.v2alpha1.ReflectionService/GetQueryServicesDescriptor",
            get_query_services_descriptor_request,
            GetQueryServicesDescriptorResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def get_tx_descriptor(
        self,
        get_tx_descriptor_request: "GetTxDescriptorRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "GetTxDescriptorResponse":
        return await self._unary_unary(
            "/cosmos.base.reflection.v2alpha1.ReflectionService/GetTxDescriptor",
            get_tx_descriptor_request,
            GetTxDescriptorResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class ReflectionServiceBase(ServiceBase):
    async def get_authn_descriptor(
        self, get_authn_descriptor_request: "GetAuthnDescriptorRequest"
    ) -> "GetAuthnDescriptorResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_chain_descriptor(
        self, get_chain_descriptor_request: "GetChainDescriptorRequest"
    ) -> "GetChainDescriptorResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_codec_descriptor(
        self, get_codec_descriptor_request: "GetCodecDescriptorRequest"
    ) -> "GetCodecDescriptorResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_configuration_descriptor(
        self, get_configuration_descriptor_request: "GetConfigurationDescriptorRequest"
    ) -> "GetConfigurationDescriptorResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_query_services_descriptor(
        self, get_query_services_descriptor_request: "GetQueryServicesDescriptorRequest"
    ) -> "GetQueryServicesDescriptorResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def get_tx_descriptor(
        self, get_tx_descriptor_request: "GetTxDescriptorRequest"
    ) -> "GetTxDescriptorResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_get_authn_descriptor(
        self,
        stream: "grpclib.server.Stream[GetAuthnDescriptorRequest, GetAuthnDescriptorResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_authn_descriptor(request)
        await stream.send_message(response)

    async def __rpc_get_chain_descriptor(
        self,
        stream: "grpclib.server.Stream[GetChainDescriptorRequest, GetChainDescriptorResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_chain_descriptor(request)
        await stream.send_message(response)

    async def __rpc_get_codec_descriptor(
        self,
        stream: "grpclib.server.Stream[GetCodecDescriptorRequest, GetCodecDescriptorResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_codec_descriptor(request)
        await stream.send_message(response)

    async def __rpc_get_configuration_descriptor(
        self,
        stream: "grpclib.server.Stream[GetConfigurationDescriptorRequest, GetConfigurationDescriptorResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_configuration_descriptor(request)
        await stream.send_message(response)

    async def __rpc_get_query_services_descriptor(
        self,
        stream: "grpclib.server.Stream[GetQueryServicesDescriptorRequest, GetQueryServicesDescriptorResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_query_services_descriptor(request)
        await stream.send_message(response)

    async def __rpc_get_tx_descriptor(
        self,
        stream: "grpclib.server.Stream[GetTxDescriptorRequest, GetTxDescriptorResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.get_tx_descriptor(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.base.reflection.v2alpha1.ReflectionService/GetAuthnDescriptor": grpclib.const.Handler(
                self.__rpc_get_authn_descriptor,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetAuthnDescriptorRequest,
                GetAuthnDescriptorResponse,
            ),
            "/cosmos.base.reflection.v2alpha1.ReflectionService/GetChainDescriptor": grpclib.const.Handler(
                self.__rpc_get_chain_descriptor,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetChainDescriptorRequest,
                GetChainDescriptorResponse,
            ),
            "/cosmos.base.reflection.v2alpha1.ReflectionService/GetCodecDescriptor": grpclib.const.Handler(
                self.__rpc_get_codec_descriptor,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetCodecDescriptorRequest,
                GetCodecDescriptorResponse,
            ),
            "/cosmos.base.reflection.v2alpha1.ReflectionService/GetConfigurationDescriptor": grpclib.const.Handler(
                self.__rpc_get_configuration_descriptor,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetConfigurationDescriptorRequest,
                GetConfigurationDescriptorResponse,
            ),
            "/cosmos.base.reflection.v2alpha1.ReflectionService/GetQueryServicesDescriptor": grpclib.const.Handler(
                self.__rpc_get_query_services_descriptor,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetQueryServicesDescriptorRequest,
                GetQueryServicesDescriptorResponse,
            ),
            "/cosmos.base.reflection.v2alpha1.ReflectionService/GetTxDescriptor": grpclib.const.Handler(
                self.__rpc_get_tx_descriptor,
                grpclib.const.Cardinality.UNARY_UNARY,
                GetTxDescriptorRequest,
                GetTxDescriptorResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/tx/signing/v1beta1/signing.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf

from ....crypto.multisig import v1beta1 as ___crypto_multisig_v1_beta1__


class SignMode(betterproto.Enum):
    """SignMode represents a signing mode with its own security guarantees."""

    SIGN_MODE_UNSPECIFIED = 0
    """
    SIGN_MODE_UNSPECIFIED specifies an unknown signing mode and will be
    rejected
    """

    SIGN_MODE_DIRECT = 1
    """
    SIGN_MODE_DIRECT specifies a signing mode which uses SignDoc and is
    verified with raw bytes from Tx
    """

    SIGN_MODE_TEXTUAL = 2
    """
    SIGN_MODE_TEXTUAL is a future signing mode that will verify some human-
    readable textual representation on top of the binary representation from
    SIGN_MODE_DIRECT
    """

    SIGN_MODE_LEGACY_AMINO_JSON = 127
    """
    SIGN_MODE_LEGACY_AMINO_JSON is a backwards compatibility mode which uses
    Amino JSON and will be removed in the future
    """

    SIGN_MODE_EIP_191 = 191
    """
    SIGN_MODE_EIP_191 specifies the sign mode for EIP 191 signing on the Cosmos
    SDK. Ref: https://eips.ethereum.org/EIPS/eip-191 Currently,
    SIGN_MODE_EIP_191 is registered as a SignMode enum variant, but is not
    implemented on the SDK by default. To enable EIP-191, you need to pass a
    custom `TxConfig` that has an implementation of `SignModeHandler` for
    EIP-191. The SDK may decide to fully support EIP-191 in the future. Since:
    cosmos-sdk 0.45.2
    """


@dataclass(eq=False, repr=False)
class SignatureDescriptors(betterproto.Message):
    """SignatureDescriptors wraps multiple SignatureDescriptor's."""

    signatures: List["SignatureDescriptor"] = betterproto.message_field(1)
    """signatures are the signature descriptors"""


@dataclass(eq=False, repr=False)
class SignatureDescriptor(betterproto.Message):
    """
    SignatureDescriptor is a convenience type which represents the full data
    for a signature including the public key of the signer, signing modes and
    the signature itself. It is primarily used for coordinating signatures
    between clients.
    """

    public_key: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(1)
    """public_key is the public key of the signer"""

    data: "SignatureDescriptorData" = betterproto.message_field(2)
    sequence: int = betterproto.uint64_field(3)
    """
    sequence is the sequence of the account, which describes the number of
    committed transactions signed by a given address. It is used to prevent
    replay attacks.
    """


@dataclass(eq=False, repr=False)
class SignatureDescriptorData(betterproto.Message):
    """Data represents signature data"""

    single: "SignatureDescriptorDataSingle" = betterproto.message_field(1, group="sum")
    """single represents a single signer"""

    multi: "SignatureDescriptorDataMulti" = betterproto.message_field(2, group="sum")
    """multi represents a multisig signer"""


@dataclass(eq=False, repr=False)
class SignatureDescriptorDataSingle(betterproto.Message):
    """Single is the signature data for a single signer"""

    mode: "SignMode" = betterproto.enum_field(1)
    """mode is the signing mode of the single signer"""

    signature: bytes = betterproto.bytes_field(2)
    """signature is the raw signature bytes"""


@dataclass(eq=False, repr=False)
class SignatureDescriptorDataMulti(betterproto.Message):
    """Multi is the signature data for a multisig public key"""

    bitarray: "___crypto_multisig_v1_beta1__.CompactBitArray" = betterproto.message_field(1)
    """bitarray specifies which keys within the multisig are signing"""

    signatures: List["SignatureDescriptorData"] = betterproto.message_field(2)
    """signatures is the signatures of the multi-signature"""



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: tendermint/version/types.proto
# plugin: python-betterproto
from dataclasses import dataclass

import betterproto


@dataclass(eq=False, repr=False)
class App(betterproto.Message):
    """
    App includes the protocol and software version for the application. This
    information is included in ResponseInfo. The App.Protocol can be updated in
    ResponseEndBlock.
    """

    protocol: int = betterproto.uint64_field(1)
    software: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class Consensus(betterproto.Message):
    """
    Consensus captures the consensus rules for processing a block in the
    blockchain, including all blockchain data structures and the rules of the
    application's state transition machine.
    """

    block: int = betterproto.uint64_field(1)
    app: int = betterproto.uint64_field(2)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/slashing/v1beta1/genesis.proto, cosmos/slashing/v1beta1/query.proto, cosmos/slashing/v1beta1/slashing.proto, cosmos/slashing/v1beta1/tx.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import (
    datetime,
    timedelta,
)
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ...base.query import v1beta1 as __base_query_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class MsgUnjail(betterproto.Message):
    """MsgUnjail defines the Msg/Unjail request type"""

    validator_addr: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class MsgUnjailResponse(betterproto.Message):
    """MsgUnjailResponse defines the Msg/Unjail response type"""

    pass


@dataclass(eq=False, repr=False)
class ValidatorSigningInfo(betterproto.Message):
    """
    ValidatorSigningInfo defines a validator's signing info for monitoring
    their liveness activity.
    """

    address: str = betterproto.string_field(1)
    start_height: int = betterproto.int64_field(2)
    """Height at which validator was first a candidate OR was unjailed"""

    index_offset: int = betterproto.int64_field(3)
    """
    Index which is incremented each time the validator was a bonded in a block
    and may have signed a precommit or not. This in conjunction with the
    `SignedBlocksWindow` param determines the index in the
    `MissedBlocksBitArray`.
    """

    jailed_until: datetime = betterproto.message_field(4)
    """
    Timestamp until which the validator is jailed due to liveness downtime.
    """

    tombstoned: bool = betterproto.bool_field(5)
    """
    Whether or not a validator has been tombstoned (killed out of validator
    set). It is set once the validator commits an equivocation or for any other
    configured misbehiavor.
    """

    missed_blocks_counter: int = betterproto.int64_field(6)
    """
    A counter kept to avoid unnecessary array reads. Note that
    `Sum(MissedBlocksBitArray)` always equals `MissedBlocksCounter`.
    """


@dataclass(eq=False, repr=False)
class Params(betterproto.Message):
    """Params represents the parameters used for by the slashing module."""

    signed_blocks_window: int = betterproto.int64_field(1)
    min_signed_per_window: bytes = betterproto.bytes_field(2)
    downtime_jail_duration: timedelta = betterproto.message_field(3)
    slash_fraction_double_sign: bytes = betterproto.bytes_field(4)
    slash_fraction_downtime: bytes = betterproto.bytes_field(5)


@dataclass(eq=False, repr=False)
class QueryParamsRequest(betterproto.Message):
    """
    QueryParamsRequest is the request type for the Query/Params RPC method
    """

    pass


@dataclass(eq=False, repr=False)
class QueryParamsResponse(betterproto.Message):
    """
    QueryParamsResponse is the response type for the Query/Params RPC method
    """

    params: "Params" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class QuerySigningInfoRequest(betterproto.Message):
    """
    QuerySigningInfoRequest is the request type for the Query/SigningInfo RPC
    method
    """

    cons_address: str = betterproto.string_field(1)
    """cons_address is the address to query signing info of"""


@dataclass(eq=False, repr=False)
class QuerySigningInfoResponse(betterproto.Message):
    """
    QuerySigningInfoResponse is the response type for the Query/SigningInfo RPC
    method
    """

    val_signing_info: "ValidatorSigningInfo" = betterproto.message_field(1)
    """val_signing_info is the signing info of requested val cons address"""


@dataclass(eq=False, repr=False)
class QuerySigningInfosRequest(betterproto.Message):
    """
    QuerySigningInfosRequest is the request type for the Query/SigningInfos RPC
    method
    """

    pagination: "__base_query_v1_beta1__.PageRequest" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class QuerySigningInfosResponse(betterproto.Message):
    """
    QuerySigningInfosResponse is the response type for the Query/SigningInfos
    RPC method
    """

    info: List["ValidatorSigningInfo"] = betterproto.message_field(1)
    """info is the signing info of all validators"""

    pagination: "__base_query_v1_beta1__.PageResponse" = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the slashing module's genesis state."""

    params: "Params" = betterproto.message_field(1)
    """params defines all the paramaters of related to deposit."""

    signing_infos: List["SigningInfo"] = betterproto.message_field(2)
    """
    signing_infos represents a map between validator addresses and their
    signing infos.
    """

    missed_blocks: List["ValidatorMissedBlocks"] = betterproto.message_field(3)
    """
    missed_blocks represents a map between validator addresses and their missed
    blocks.
    """


@dataclass(eq=False, repr=False)
class SigningInfo(betterproto.Message):
    """SigningInfo stores validator signing info of corresponding address."""

    address: str = betterproto.string_field(1)
    """address is the validator address."""

    validator_signing_info: "ValidatorSigningInfo" = betterproto.message_field(2)
    """
    validator_signing_info represents the signing info of this validator.
    """


@dataclass(eq=False, repr=False)
class ValidatorMissedBlocks(betterproto.Message):
    """
    ValidatorMissedBlocks contains array of missed blocks of corresponding
    address.
    """

    address: str = betterproto.string_field(1)
    """address is the validator address."""

    missed_blocks: List["MissedBlock"] = betterproto.message_field(2)
    """missed_blocks is an array of missed blocks by the validator."""


@dataclass(eq=False, repr=False)
class MissedBlock(betterproto.Message):
    """MissedBlock contains height and missed status as boolean."""

    index: int = betterproto.int64_field(1)
    """index is the height at which the block was missed."""

    missed: bool = betterproto.bool_field(2)
    """missed is the missed status."""


class MsgStub(betterproto.ServiceStub):
    async def unjail(
        self,
        msg_unjail: "MsgUnjail",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgUnjailResponse":
        return await self._unary_unary(
            "/cosmos.slashing.v1beta1.Msg/Unjail",
            msg_unjail,
            MsgUnjailResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryStub(betterproto.ServiceStub):
    async def params(
        self,
        query_params_request: "QueryParamsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryParamsResponse":
        return await self._unary_unary(
            "/cosmos.slashing.v1beta1.Query/Params",
            query_params_request,
            QueryParamsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def signing_info(
        self,
        query_signing_info_request: "QuerySigningInfoRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QuerySigningInfoResponse":
        return await self._unary_unary(
            "/cosmos.slashing.v1beta1.Query/SigningInfo",
            query_signing_info_request,
            QuerySigningInfoResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def signing_infos(
        self,
        query_signing_infos_request: "QuerySigningInfosRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QuerySigningInfosResponse":
        return await self._unary_unary(
            "/cosmos.slashing.v1beta1.Query/SigningInfos",
            query_signing_infos_request,
            QuerySigningInfosResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MsgBase(ServiceBase):
    async def unjail(self, msg_unjail: "MsgUnjail") -> "MsgUnjailResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_unjail(self, stream: "grpclib.server.Stream[MsgUnjail, MsgUnjailResponse]") -> None:
        request = await stream.recv_message()
        response = await self.unjail(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.slashing.v1beta1.Msg/Unjail": grpclib.const.Handler(
                self.__rpc_unjail,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgUnjail,
                MsgUnjailResponse,
            ),
        }


class QueryBase(ServiceBase):
    async def params(self, query_params_request: "QueryParamsRequest") -> "QueryParamsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def signing_info(self, query_signing_info_request: "QuerySigningInfoRequest") -> "QuerySigningInfoResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def signing_infos(
        self, query_signing_infos_request: "QuerySigningInfosRequest"
    ) -> "QuerySigningInfosResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_params(self, stream: "grpclib.server.Stream[QueryParamsRequest, QueryParamsResponse]") -> None:
        request = await stream.recv_message()
        response = await self.params(request)
        await stream.send_message(response)

    async def __rpc_signing_info(
        self,
        stream: "grpclib.server.Stream[QuerySigningInfoRequest, QuerySigningInfoResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.signing_info(request)
        await stream.send_message(response)

    async def __rpc_signing_infos(
        self,
        stream: "grpclib.server.Stream[QuerySigningInfosRequest, QuerySigningInfosResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.signing_infos(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.slashing.v1beta1.Query/Params": grpclib.const.Handler(
                self.__rpc_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryParamsRequest,
                QueryParamsResponse,
            ),
            "/cosmos.slashing.v1beta1.Query/SigningInfo": grpclib.const.Handler(
                self.__rpc_signing_info,
                grpclib.const.Cardinality.UNARY_UNARY,
                QuerySigningInfoRequest,
                QuerySigningInfoResponse,
            ),
            "/cosmos.slashing.v1beta1.Query/SigningInfos": grpclib.const.Handler(
                self.__rpc_signing_infos,
                grpclib.const.Cardinality.UNARY_UNARY,
                QuerySigningInfosRequest,
                QuerySigningInfosResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: oracle/v1/genesis.proto, oracle/v1/oracle.proto, oracle/v1/query.proto, oracle/v1/tx.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ...cosmos.base import v1beta1 as __cosmos_base_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


class ResolveStatus(betterproto.Enum):
    """ResolveStatus encodes the status of an oracle request."""

    RESOLVE_STATUS_OPEN_UNSPECIFIED = 0
    """Open - the request is not yet resolved."""

    RESOLVE_STATUS_SUCCESS = 1
    """Success - the request has been resolved successfully with no errors."""

    RESOLVE_STATUS_FAILURE = 2
    """Failure - an error occured during the request's resolve call."""

    RESOLVE_STATUS_EXPIRED = 3
    """
    Expired - the request does not get enough reports from validator within the
    timeframe.
    """


@dataclass(eq=False, repr=False)
class DataSource(betterproto.Message):
    """
    DataSource is the data structure for storing data sources in the storage.
    """

    owner: str = betterproto.string_field(1)
    """Owner is an address of the account who own the data source"""

    name: str = betterproto.string_field(2)
    """Name is data source name used for display"""

    description: str = betterproto.string_field(3)
    """Description is data source description used for display"""

    filename: str = betterproto.string_field(4)
    """
    Filename is string of file name used as reference for locating data source
    file stored in bandchain nodes
    """

    treasury: str = betterproto.string_field(5)
    """
    Treasury is the account address who receive data source fee from requester.
    """

    fee: List["__cosmos_base_v1_beta1__.Coin"] = betterproto.message_field(6)
    """
    Fee is the data source fee per ask_count that data provider will receive
    from requester.
    """


@dataclass(eq=False, repr=False)
class OracleScript(betterproto.Message):
    """
    OracleScript is the data structure for storing oracle scripts in the
    storage.
    """

    owner: str = betterproto.string_field(1)
    """Owner is an address of the account who own the oracle script"""

    name: str = betterproto.string_field(2)
    """Name is oracle script name used for display"""

    description: str = betterproto.string_field(3)
    """Description is oracle script description used for display"""

    filename: str = betterproto.string_field(4)
    """
    Filename is string of file name used as reference for locating compiled
    oracle script WASM file stored in bandchain nodes
    """

    schema: str = betterproto.string_field(5)
    """
    Schema is the schema of the oracle script input/output which is formatted
    in OBI format e.g. "{symbol:string,multiplier:u64}/{px:u64}"
    """

    source_code_url: str = betterproto.string_field(6)
    """
    SourceCodeURL is the URL of oracle script's source code. It is recommendded
    to store source code on IPFS and get its URL to preserve decentralization.
    """


@dataclass(eq=False, repr=False)
class RawRequest(betterproto.Message):
    """
    RawRequest is the data structure for storing raw requests in the storage.
    """

    external_id: int = betterproto.uint64_field(1)
    """ExternalID is an ID of the raw request"""

    data_source_id: int = betterproto.uint64_field(2)
    """
    DataSourceID is an ID of data source script that relates to the raw request
    """

    calldata: bytes = betterproto.bytes_field(3)
    """
    Calldata is the data used as argument params for executing data source
    script
    """


@dataclass(eq=False, repr=False)
class RawReport(betterproto.Message):
    """
    RawRequest is the data structure for storing raw reporter in the storage.
    """

    external_id: int = betterproto.uint64_field(1)
    """ExternalID is an ID of the raw request"""

    exit_code: int = betterproto.uint32_field(2)
    """
    ExitCode is status code provided by validators to specify error, if any.
    Exit code is usually filled by the exit code returned from execution of
    specified data source script. With code 0 means there is no error.
    """

    data: bytes = betterproto.bytes_field(3)
    """
    Data is raw result provided by validators. It is usually filled by the
    result from execution of specified data source script.
    """


@dataclass(eq=False, repr=False)
class Request(betterproto.Message):
    """Request is the data structure for storing requests in the storage."""

    oracle_script_id: int = betterproto.uint64_field(1)
    """OracleScriptID is ID of an oracle script"""

    calldata: bytes = betterproto.bytes_field(2)
    """Calldata is the data used as argument params for the oracle script"""

    requested_validators: List[str] = betterproto.string_field(3)
    """
    RequestedValidators is a list of validator addresses that are assigned for
    fulfilling the request
    """

    min_count: int = betterproto.uint64_field(4)
    """
    MinCount is minimum number of validators required for fulfilling the
    request
    """

    request_height: int = betterproto.int64_field(5)
    """RequestHeight is block height that the request has been created"""

    request_time: int = betterproto.int64_field(6)
    """
    RequestTime is timestamp of the chain's block which contains the request
    """

    client_id: str = betterproto.string_field(7)
    """
    ClientID is arbitrary id provided by requester. It is used by client-side
    for referencing the request
    """

    raw_requests: List["RawRequest"] = betterproto.message_field(8)
    """
    RawRequests is a list of raw requests specified by execution of oracle
    script
    """

    ibc_channel: "IbcChannel" = betterproto.message_field(9)
    """
    IBCChannel is an IBC channel info of the other chain, which contains a
    channel and a port to allow bandchain connect to that chain. This field
    allows other chain be able to request data from bandchain via IBC.
    """

    execute_gas: int = betterproto.uint64_field(10)
    """ExecuteGas is amount of gas to reserve for executing"""


@dataclass(eq=False, repr=False)
class Report(betterproto.Message):
    """Report is the data structure for storing reports in the storage."""

    validator: str = betterproto.string_field(1)
    """Validator is a validator address who submit the report"""

    in_before_resolve: bool = betterproto.bool_field(2)
    """
    InBeforeResolve indicates whether the report is submitted before the
    request resolved
    """

    raw_reports: List["RawReport"] = betterproto.message_field(3)
    """
    RawReports is list of raw reports provided by the validator. Each raw
    report has different external ID
    """


@dataclass(eq=False, repr=False)
class OracleRequestPacketData(betterproto.Message):
    """
    OracleRequestPacketData encodes an oracle request sent from other
    blockchains to BandChain.
    """

    client_id: str = betterproto.string_field(1)
    """
    ClientID is the unique identifier of this oracle request, as specified by
    the client. This same unique ID will be sent back to the requester with the
    oracle response.
    """

    oracle_script_id: int = betterproto.uint64_field(2)
    """
    OracleScriptID is the unique identifier of the oracle script to be
    executed.
    """

    calldata: bytes = betterproto.bytes_field(3)
    """
    Calldata is the OBI-encoded calldata bytes available for oracle executor to
    read.
    """

    ask_count: int = betterproto.uint64_field(4)
    """
    AskCount is the number of validators that are requested to respond to this
    oracle request. Higher value means more security, at a higher gas cost.
    """

    min_count: int = betterproto.uint64_field(5)
    """
    MinCount is the minimum number of validators necessary for the request to
    proceed to the execution phase. Higher value means more security, at the
    cost of liveness.
    """

    fee_limit: List["__cosmos_base_v1_beta1__.Coin"] = betterproto.message_field(6)
    """
    FeeLimit is the maximum tokens that will be paid to all data source
    providers.
    """

    prepare_gas: int = betterproto.uint64_field(7)
    """PrepareGas is amount of gas to pay to prepare raw requests"""

    execute_gas: int = betterproto.uint64_field(8)
    """ExecuteGas is amount of gas to reserve for executing"""


@dataclass(eq=False, repr=False)
class OracleRequestPacketAcknowledgement(betterproto.Message):
    """
    OracleRequestPacketAcknowledgement encodes an oracle request
    acknowledgement send back to requester chain.
    """

    request_id: int = betterproto.uint64_field(1)
    """RequestID is BandChain's unique identifier for this oracle request."""


@dataclass(eq=False, repr=False)
class OracleResponsePacketData(betterproto.Message):
    """
    OracleResponsePacketData encodes an oracle response from BandChain to the
    requester.
    """

    client_id: str = betterproto.string_field(1)
    """
    ClientID is the unique identifier matched with that of the oracle request
    packet.
    """

    request_id: int = betterproto.uint64_field(2)
    """RequestID is BandChain's unique identifier for this oracle request."""

    ans_count: int = betterproto.uint64_field(3)
    """
    AnsCount is the number of validators among to the asked validators that
    actually responded to this oracle request prior to this oracle request
    being resolved.
    """

    request_time: int = betterproto.int64_field(4)
    """
    RequestTime is the UNIX epoch time at which the request was sent to
    BandChain.
    """

    resolve_time: int = betterproto.int64_field(5)
    """
    ResolveTime is the UNIX epoch time at which the request was resolved to the
    final result.
    """

    resolve_status: "ResolveStatus" = betterproto.enum_field(6)
    """
    ResolveStatus is the status of this oracle request, which can be OK,
    FAILURE, or EXPIRED.
    """

    result: bytes = betterproto.bytes_field(7)
    """
    Result is the final aggregated value encoded in OBI format. Only available
    if status if OK.
    """


@dataclass(eq=False, repr=False)
class Result(betterproto.Message):
    """Result encodes a result of request and store in chain"""

    client_id: str = betterproto.string_field(1)
    """
    ClientID is the unique identifier of this oracle request, as specified by
    the client. This same unique ID will be sent back to the requester with the
    oracle response.
    """

    oracle_script_id: int = betterproto.uint64_field(2)
    """
    OracleScriptID is the unique identifier of the oracle script to be
    executed.
    """

    calldata: bytes = betterproto.bytes_field(3)
    """
    Calldata is the calldata bytes available for oracle executor to read.
    """

    ask_count: int = betterproto.uint64_field(4)
    """
    AskCount is the number of validators that are requested to respond to this
    oracle request. Higher value means more security, at a higher gas cost.
    """

    min_count: int = betterproto.uint64_field(5)
    """
    MinCount is the minimum number of validators necessary for the request to
    proceed to the execution phase. Higher value means more security, at the
    cost of liveness.
    """

    request_id: int = betterproto.uint64_field(6)
    """RequestID is BandChain's unique identifier for this oracle request."""

    ans_count: int = betterproto.uint64_field(7)
    """
    AnsCount is the number of validators among to the asked validators that
    actually responded to this oracle request prior to this oracle request
    being resolved.
    """

    request_time: int = betterproto.int64_field(8)
    """
    RequestTime is the UNIX epoch time at which the request was sent to
    BandChain.
    """

    resolve_time: int = betterproto.int64_field(9)
    """
    ResolveTime is the UNIX epoch time at which the request was resolved to the
    final result.
    """

    resolve_status: "ResolveStatus" = betterproto.enum_field(10)
    """
    ResolveStatus is the status of this oracle request, which can be OK,
    FAILURE, or EXPIRED.
    """

    result: bytes = betterproto.bytes_field(11)
    """Result is the final aggregated value only available if status if OK."""


@dataclass(eq=False, repr=False)
class ValidatorStatus(betterproto.Message):
    """
    ValidatorStatus maintains whether a validator is an active oracle provider.
    """

    is_active: bool = betterproto.bool_field(1)
    """
    IsActive is a boolean indicating active status of validator. The validator
    will be deactivated when they are unable to send reports to fulfill oracle
    request before the request expired.
    """

    since: datetime = betterproto.message_field(2)
    """
    Since is a block timestamp when validator has been activated/deactivated
    """


@dataclass(eq=False, repr=False)
class ActiveValidator(betterproto.Message):
    """ActiveValidator is information of currently active validator"""

    address: str = betterproto.string_field(1)
    """Address is a validator address"""

    power: int = betterproto.uint64_field(2)
    """Power is an amount of token that the validator is holding"""


@dataclass(eq=False, repr=False)
class Params(betterproto.Message):
    """
    Params is the data structure that keeps the parameters of the oracle
    module.
    """

    max_raw_request_count: int = betterproto.uint64_field(1)
    """
    MaxRawRequestCount is the maximum number of data source raw requests a
    request can make.
    """

    max_ask_count: int = betterproto.uint64_field(2)
    """
    MaxAskCount is the maximum number of validators a request can target.
    """

    max_calldata_size: int = betterproto.uint64_field(3)
    """
    MaxCalldataSize is the maximum size limit of calldata (bytes) in a request.
    """

    max_report_data_size: int = betterproto.uint64_field(4)
    """
    MaxReportDataSize is the maximum size limit of report data (bytes) in a
    report.
    """

    expiration_block_count: int = betterproto.uint64_field(5)
    """
    ExpirationBlockCount is the number of blocks a request stays valid before
    it gets expired due to insufficient reports.
    """

    base_owasm_gas: int = betterproto.uint64_field(6)
    """
    BaseOwasmGas is the base amount of Cosmos-SDK gas charged for owasm
    execution.
    """

    per_validator_request_gas: int = betterproto.uint64_field(7)
    """
    PerValidatorRequestGas is the amount of Cosmos-SDK gas charged per
    requested validator.
    """

    sampling_try_count: int = betterproto.uint64_field(8)
    """
    SamplingTryCount the number of validator sampling tries to pick the highest
    voting power subset of validators to perform an oracle task.
    """

    oracle_reward_percentage: int = betterproto.uint64_field(9)
    """
    OracleRewardPercentage is the percentage of block rewards allocated to
    active oracle validators.
    """

    inactive_penalty_duration: int = betterproto.uint64_field(10)
    """
    InactivePenaltyDuration is the duration period where a validator cannot
    activate back after missing an oracle report.
    """

    ibc_request_enabled: bool = betterproto.bool_field(11)
    """
    IBCRequestEnabled is a flag indicating whether sending oracle request via
    IBC is allowed
    """


@dataclass(eq=False, repr=False)
class PendingResolveList(betterproto.Message):
    """
    PendingResolveList is a list of requests that are waiting to be resolved
    """

    request_ids: List[int] = betterproto.uint64_field(1)
    """RequestIDs is a list of request IDs that are waiting to be resolved"""


@dataclass(eq=False, repr=False)
class IbcChannel(betterproto.Message):
    """
    IBCChannel is information of IBC protocol to allow communicating with other
    chain
    """

    port_id: str = betterproto.string_field(1)
    """
    PortID is port ID used for sending response packet when request is
    resolved.
    """

    channel_id: str = betterproto.string_field(2)
    """
    ChannelID is channel ID used for sending response packet when request is
    resolved.
    """


@dataclass(eq=False, repr=False)
class RequestVerification(betterproto.Message):
    """
    RequestVerification is a message that is constructed and signed by a
    reporter to be used as a part of verification of oracle request.
    """

    chain_id: str = betterproto.string_field(1)
    """ChainID is the ID of targeted chain"""

    validator: str = betterproto.string_field(2)
    """Validator is an validator address"""

    request_id: int = betterproto.uint64_field(3)
    """RequestID is the targeted request ID"""

    external_id: int = betterproto.uint64_field(4)
    """ExternalID is the oracle's external ID of data source"""

    data_source_id: int = betterproto.uint64_field(5)
    """DataSourceID is the ID of data source"""


@dataclass(eq=False, repr=False)
class PriceResult(betterproto.Message):
    """PriceResult is a result from standard price reference"""

    symbol: str = betterproto.string_field(1)
    """
    Symbol is unit of data indicating what the data is. It is price currencies
    for this case.
    """

    multiplier: int = betterproto.uint64_field(2)
    """
    Multiplier is a number used for left-shifting value to eliminate decimal
    digits
    """

    px: int = betterproto.uint64_field(3)
    """
    Px is the actual data, which is rate number multiplied by the multiplier.
    """

    request_id: int = betterproto.uint64_field(4)
    """RequestID is oracle request ID that contains this price"""

    resolve_time: int = betterproto.int64_field(5)
    """
    ResolveTime is epoch timestamp indicating the time when the request had
    been resolved
    """


@dataclass(eq=False, repr=False)
class MsgRequestData(betterproto.Message):
    """MsgRequestData is a message for sending a data oracle request."""

    oracle_script_id: int = betterproto.uint64_field(1)
    """OracleScriptID is the identifier of the oracle script to be called."""

    calldata: bytes = betterproto.bytes_field(2)
    """Calldata is the OBI-encoded call parameters for the oracle script."""

    ask_count: int = betterproto.uint64_field(3)
    """AskCount is the number of validators to perform the oracle task."""

    min_count: int = betterproto.uint64_field(4)
    """
    MinCount is the minimum number of validators sufficient to resolve the
    oracle tasks.
    """

    client_id: str = betterproto.string_field(5)
    """
    ClientID is the client-provided unique identifier to track the request.
    """

    fee_limit: List["__cosmos_base_v1_beta1__.Coin"] = betterproto.message_field(6)
    """
    FeeLimit is the maximum tokens that will be paid to all data source
    providers.
    """

    prepare_gas: int = betterproto.uint64_field(7)
    """PrepareGas is amount of gas to pay to prepare raw requests"""

    execute_gas: int = betterproto.uint64_field(8)
    """ExecuteGas is amount of gas to reserve for executing"""

    sender: str = betterproto.string_field(9)
    """Sender is an account address of message sender."""


@dataclass(eq=False, repr=False)
class MsgRequestDataResponse(betterproto.Message):
    """MsgRequestDataResponse is response data for MsgRequestData message"""

    pass


@dataclass(eq=False, repr=False)
class MsgReportData(betterproto.Message):
    """
    MsgReportData is a message for reporting to a data request by a validator.
    """

    request_id: int = betterproto.uint64_field(1)
    """RequestID is the identifier of the request to be reported to."""

    raw_reports: List["RawReport"] = betterproto.message_field(2)
    """
    RawReports is the list of report information provided by data sources
    identified by external ID
    """

    validator: str = betterproto.string_field(3)
    """Validator is the address of the validator that owns this report."""


@dataclass(eq=False, repr=False)
class MsgReportDataResponse(betterproto.Message):
    """MsgReportDataResponse is response data for MsgReportData message"""

    pass


@dataclass(eq=False, repr=False)
class MsgCreateDataSource(betterproto.Message):
    """MsgCreateDataSource is a message for creating a new data source."""

    name: str = betterproto.string_field(1)
    """Name is the name of this data source used for display (optional)."""

    description: str = betterproto.string_field(2)
    """
    Description is the description of this data source used for display
    (optional).
    """

    executable: bytes = betterproto.bytes_field(3)
    """
    Executable is the content of executable script or binary file to be run by
    validators upon execution.
    """

    fee: List["__cosmos_base_v1_beta1__.Coin"] = betterproto.message_field(4)
    """
    Fee is the data source fee per ask_count that data provider will receive
    from requester.
    """

    treasury: str = betterproto.string_field(5)
    """
    Treasury is the account address who receive data source fee from requester.
    """

    owner: str = betterproto.string_field(6)
    """
    Owner is the account address who is allowed to make further changes to the
    data source.
    """

    sender: str = betterproto.string_field(7)
    """Sender is the signer of this message."""


@dataclass(eq=False, repr=False)
class MsgCreateDataSourceResponse(betterproto.Message):
    """
    MsgCreateDataSourceResponse is response data for MsgCreateDataSource
    message
    """

    pass


@dataclass(eq=False, repr=False)
class MsgEditDataSource(betterproto.Message):
    """MsgEditDataSource is a message for editing an existing data source."""

    data_source_id: int = betterproto.uint64_field(1)
    """
    DataSourceID is the unique identifier of the data source to be edited.
    """

    name: str = betterproto.string_field(2)
    """Name is the name of this data source used for display (optional)."""

    description: str = betterproto.string_field(3)
    """
    Description is the description of this data source used for display
    (optional).
    """

    executable: bytes = betterproto.bytes_field(4)
    """
    Executable is the executable script or binary to be run by validators upon
    execution.
    """

    fee: List["__cosmos_base_v1_beta1__.Coin"] = betterproto.message_field(5)
    """
    Fee is the data source fee per ask_count that data provider will receive
    from requester.
    """

    treasury: str = betterproto.string_field(6)
    """Treasury is the address who receive data source fee from requester."""

    owner: str = betterproto.string_field(7)
    """
    Owner is the address who is allowed to make further changes to the data
    source.
    """

    sender: str = betterproto.string_field(8)
    """
    Sender is the signer of this message. Must be the current data source's
    owner.
    """


@dataclass(eq=False, repr=False)
class MsgEditDataSourceResponse(betterproto.Message):
    """
    MsgEditDataSourceResponse is response data for MsgEditDataSource message
    """

    pass


@dataclass(eq=False, repr=False)
class MsgCreateOracleScript(betterproto.Message):
    """MsgCreateOracleScript is a message for creating an oracle script."""

    name: str = betterproto.string_field(1)
    """Name is the name of this oracle script used for display (optional)."""

    description: str = betterproto.string_field(2)
    """
    Description is the description of this oracle script used for display
    (optional).
    """

    schema: str = betterproto.string_field(3)
    """Schema is the OBI schema of this oracle script (optional)."""

    source_code_url: str = betterproto.string_field(4)
    """
    SourceCodeURL is the absolute URI to the script's source code (optional).
    """

    code: bytes = betterproto.bytes_field(5)
    """
    Code is the oracle WebAssembly binary code. Can be raw of gzip compressed.
    """

    owner: str = betterproto.string_field(6)
    """
    Owner is the address who is allowed to make further changes to the oracle
    script.
    """

    sender: str = betterproto.string_field(7)
    """Sender is the signer of this message."""


@dataclass(eq=False, repr=False)
class MsgCreateOracleScriptResponse(betterproto.Message):
    """
    MsgCreateOracleScriptResponse is response data for MsgCreateOracleScript
    message
    """

    pass


@dataclass(eq=False, repr=False)
class MsgEditOracleScript(betterproto.Message):
    """
    MsgEditOracleScript is a message for editing an existing oracle script.
    """

    oracle_script_id: int = betterproto.uint64_field(1)
    """
    OracleScriptID is the unique identifier of the oracle script to be edited.
    """

    name: str = betterproto.string_field(2)
    """Name is the name of this oracle script used for display (optional)."""

    description: str = betterproto.string_field(3)
    """
    Description is the description of this oracle script used for display
    (optional).
    """

    schema: str = betterproto.string_field(4)
    """Schema is the OBI schema of this oracle script (optional)."""

    source_code_url: str = betterproto.string_field(5)
    """
    SourceCodeURL is the absolute URI to the script's source code (optional).
    """

    code: bytes = betterproto.bytes_field(6)
    """
    Code is the oracle WebAssembly binary code. Can be raw of gzip compressed.
    """

    owner: str = betterproto.string_field(7)
    """
    Owner is an account address who is allowed to make further changes to the
    oracle script.
    """

    sender: str = betterproto.string_field(8)
    """
    Sender is an account address who sign this message. Must be the current
    oracle script's owner.
    """


@dataclass(eq=False, repr=False)
class MsgEditOracleScriptResponse(betterproto.Message):
    """
    MsgEditOracleScriptResponse is response data for MsgEditOracleScript
    message
    """

    pass


@dataclass(eq=False, repr=False)
class MsgActivate(betterproto.Message):
    """
    MsgEditOracleScript is a message for activating a validator to become an
    oracle provider. However, the activation can be revoked once the validator
    is unable to provide data to fulfill requests
    """

    validator: str = betterproto.string_field(1)
    """
    Validator is the validator address who sign this message and request to be
    activated.
    """


@dataclass(eq=False, repr=False)
class MsgActivateResponse(betterproto.Message):
    """MsgActivateResponse is response data for MsgActivate message"""

    pass


@dataclass(eq=False, repr=False)
class QueryCountsRequest(betterproto.Message):
    """QueryCountsRequest is request type for the Query/Count RPC method."""

    pass


@dataclass(eq=False, repr=False)
class QueryCountsResponse(betterproto.Message):
    """QueryCountsResponse is response type for the Query/Count RPC method."""

    data_source_count: int = betterproto.uint64_field(1)
    """
    DataSourceCount is total number of data sources available on the chain
    """

    oracle_script_count: int = betterproto.uint64_field(2)
    """
    OracleScriptCount is total number of oracle scripts available on the chain
    """

    request_count: int = betterproto.uint64_field(3)
    """RequestCount is total number of requests submitted to the chain"""


@dataclass(eq=False, repr=False)
class QueryDataRequest(betterproto.Message):
    """QueryDataRequest is request type for the Query/Data RPC method."""

    data_hash: str = betterproto.string_field(1)
    """
    DataHash is SHA256 hash of the file's content, which can be data source or
    oracle script
    """


@dataclass(eq=False, repr=False)
class QueryDataResponse(betterproto.Message):
    """QueryDataResponse is response type for the Query/Data RPC method."""

    data: bytes = betterproto.bytes_field(1)
    """Data is file's content, which can be data source or oracle script"""


@dataclass(eq=False, repr=False)
class QueryDataSourceRequest(betterproto.Message):
    """
    QueryDataSourceRequest is request type for the Query/DataSource RPC method.
    """

    data_source_id: int = betterproto.uint64_field(1)
    """DataSourceID is ID of a data source script"""


@dataclass(eq=False, repr=False)
class QueryDataSourceResponse(betterproto.Message):
    """
    QueryDataSourceResponse is response type for the Query/DataSource RPC
    method.
    """

    data_source: "DataSource" = betterproto.message_field(1)
    """DataSource is summary information of a data source"""


@dataclass(eq=False, repr=False)
class QueryOracleScriptRequest(betterproto.Message):
    """
    QueryOracleScriptRequest is request type for the Query/OracleScript RPC
    method.
    """

    oracle_script_id: int = betterproto.uint64_field(1)
    """OracleScriptID is ID of an oracle script"""


@dataclass(eq=False, repr=False)
class QueryOracleScriptResponse(betterproto.Message):
    """
    QueryOracleScriptResponse is response type for the Query/OracleScript RPC
    method.
    """

    oracle_script: "OracleScript" = betterproto.message_field(1)
    """OracleScript is summary information of an oracle script"""


@dataclass(eq=False, repr=False)
class QueryRequestRequest(betterproto.Message):
    """
    QueryRequestRequest is request type for the Query/Request RPC method.
    """

    request_id: int = betterproto.uint64_field(1)
    """RequestID is ID of an oracle request"""


@dataclass(eq=False, repr=False)
class QueryRequestResponse(betterproto.Message):
    """
    QueryRequestResponse is response type for the Query/Request RPC method.
    """

    request: "Request" = betterproto.message_field(1)
    """Request is an oracle request"""

    reports: List["Report"] = betterproto.message_field(2)
    """
    Reports is list of result data as raw reports that are fulfilled by
    assigned validators
    """

    result: "Result" = betterproto.message_field(3)
    """Result is a final form of result data"""


@dataclass(eq=False, repr=False)
class QueryPendingRequestsRequest(betterproto.Message):
    """
    QueryPendingRequestRequest is request type for the Query/PendingRequests
    RPC method.
    """

    validator_address: str = betterproto.string_field(1)
    """ValidatorAddress is address of a validator"""


@dataclass(eq=False, repr=False)
class QueryPendingRequestsResponse(betterproto.Message):
    """
    QueryPendingRequestResponse is response type for the Query/PendingRequests
    RPC method.
    """

    request_ids: List[int] = betterproto.uint64_field(1)
    """
    RequestIDs is a list of pending request IDs assigned to the given validator
    """


@dataclass(eq=False, repr=False)
class QueryParamsRequest(betterproto.Message):
    """QueryParamsRequest is request type for the Query/Params RPC method."""

    pass


@dataclass(eq=False, repr=False)
class QueryParamsResponse(betterproto.Message):
    """
    QueryParamsResponse is response type for the Query/Params RPC method.
    """

    params: "Params" = betterproto.message_field(1)
    """pagination defines an optional pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryValidatorRequest(betterproto.Message):
    """
    QueryValidatorRequest is request type for the Query/Validator RPC method.
    """

    validator_address: str = betterproto.string_field(1)
    """ValidatorAddress is address of a validator"""


@dataclass(eq=False, repr=False)
class QueryValidatorResponse(betterproto.Message):
    """
    QueryValidatorResponse is response type for the Query/Validator RPC method.
    """

    status: "ValidatorStatus" = betterproto.message_field(1)
    """Status is status of a validator e.g. active/inactive"""


@dataclass(eq=False, repr=False)
class QueryIsReporterRequest(betterproto.Message):
    """
    QueryIsReporterRequest is request type for the Query/Reporter RPC method.
    """

    validator_address: str = betterproto.string_field(1)
    """ValidatorAddress is a validator address"""

    reporter_address: str = betterproto.string_field(2)
    """ReporterAddress is a candidate account"""


@dataclass(eq=False, repr=False)
class QueryIsReporterResponse(betterproto.Message):
    """
    QueryIsReporterResponse is response type for the Query/Reporter RPC method.
    """

    is_reporter: bool = betterproto.bool_field(1)
    """IsReporter is true if this account has been granted by validator"""


@dataclass(eq=False, repr=False)
class QueryReportersRequest(betterproto.Message):
    """
    QueryReportersRequest is request type for the Query/Reporters RPC method.
    """

    validator_address: str = betterproto.string_field(1)
    """ValidatorAddress is a validator address"""


@dataclass(eq=False, repr=False)
class QueryReportersResponse(betterproto.Message):
    """
    QueryReportersResponse is response type for the Query/Reporters RPC method.
    """

    reporter: List[str] = betterproto.string_field(1)
    """Reporter is a list of account addresses of reporters"""


@dataclass(eq=False, repr=False)
class QueryActiveValidatorsRequest(betterproto.Message):
    """
    QueryActiveValidatorsRequest is request type for the Query/ActiveValidators
    RPC method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryActiveValidatorsResponse(betterproto.Message):
    """
    QueryActiveValidatorsResponse is response type for the
    Query/ActiveValidators RPC method.
    """

    validators: List["ActiveValidator"] = betterproto.message_field(1)
    """Validators is a list of active validators"""


@dataclass(eq=False, repr=False)
class QueryRequestSearchRequest(betterproto.Message):
    """
    QueryRequestSearchRequest is request type for the Query/RequestSearch RPC
    method.
    """

    oracle_script_id: int = betterproto.uint64_field(1)
    """OracleScriptID is ID of an oracle script"""

    calldata: str = betterproto.string_field(2)
    """
    Calldata is OBI-encoded data in hex format as argument params for the
    oracle script
    """

    ask_count: int = betterproto.uint64_field(3)
    """AskCount is number of validators allowed for fulfilling the request"""

    min_count: int = betterproto.uint64_field(4)
    """MinCount is number of validators required for fulfilling the request"""


@dataclass(eq=False, repr=False)
class QueryRequestSearchResponse(betterproto.Message):
    """
    QueryRequestSearchResponse is response type for the Query/RequestSearch RPC
    method.
    """

    request: "QueryRequestResponse" = betterproto.message_field(1)
    """Request is details of an oracle request"""


@dataclass(eq=False, repr=False)
class QueryRequestPriceRequest(betterproto.Message):
    """
    QueryRequestPriceRequest is request type for the Query/RequestPrice RPC
    method.
    """

    symbols: List[str] = betterproto.string_field(1)
    """Symbol is unit of data indicating what the data is"""

    ask_count: int = betterproto.uint64_field(2)
    """AskCount is number of validators allowed for fulfilling the request"""

    min_count: int = betterproto.uint64_field(3)
    """MinCount is number of validators required for fulfilling the request"""


@dataclass(eq=False, repr=False)
class QueryRequestPriceResponse(betterproto.Message):
    """
    QueryRequestPriceResponse is response type for the Query/RequestPrice RPC
    method.
    """

    price_results: List["PriceResult"] = betterproto.message_field(1)
    """PriceResult is a list of price results for given symbols"""


@dataclass(eq=False, repr=False)
class QueryRequestVerificationRequest(betterproto.Message):
    """
    QueryRequestVerificationRequest is request type for the
    Query/RequestVerification RPC
    """

    chain_id: str = betterproto.string_field(1)
    """
    ChainID is the chain ID to identify which chain ID is used for the
    verification
    """

    validator: str = betterproto.string_field(2)
    """Validator is a validator address"""

    request_id: int = betterproto.uint64_field(3)
    """RequestID is oracle request ID"""

    external_id: int = betterproto.uint64_field(4)
    """ExternalID is an oracle's external ID"""

    data_source_id: int = betterproto.uint64_field(5)
    """DataSourceID is the ID of data source"""

    reporter: str = betterproto.string_field(6)
    """
    Reporter is an bech32-encoded public key of the reporter authorized by the
    validator
    """

    signature: bytes = betterproto.bytes_field(7)
    """
    Signature is a signature signed by the reporter using reporter's private
    key
    """

    max_delay: int = betterproto.uint64_field(8)
    """
    MaxDelay is a max number of RequestID delay acceptable in case of node
    delay
    """


@dataclass(eq=False, repr=False)
class QueryRequestVerificationResponse(betterproto.Message):
    """
    QueryRequestVerificationResponse is response type for the
    Query/RequestVerification RPC
    """

    chain_id: str = betterproto.string_field(1)
    """ChainID is the targeted chain ID"""

    validator: str = betterproto.string_field(2)
    """Validator is the targeted validator address"""

    request_id: int = betterproto.uint64_field(3)
    """RequestID is the ID of targeted request"""

    external_id: int = betterproto.uint64_field(4)
    """ExternalID is the ID of targeted oracle's external data source"""

    data_source_id: int = betterproto.uint64_field(5)
    """
    DataSourceID is the ID of a data source that relates to the targeted
    external ID
    """

    is_delay: bool = betterproto.bool_field(6)
    """
    IsDelay is a flag to tell that this request is not exist yet but in
    MaxDelay range
    """


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the oracle module's genesis state."""

    params: "Params" = betterproto.message_field(1)
    """Params defines all the paramaters of the module."""

    data_sources: List["DataSource"] = betterproto.message_field(2)
    """DataSources are data sources to be installed during genesis phase"""

    oracle_scripts: List["OracleScript"] = betterproto.message_field(3)
    """
    OracleScripts are list of oracle scripts to be installed during genesis
    phase
    """


class MsgStub(betterproto.ServiceStub):
    async def request_data(
        self,
        msg_request_data: "MsgRequestData",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgRequestDataResponse":
        return await self._unary_unary(
            "/oracle.v1.Msg/RequestData",
            msg_request_data,
            MsgRequestDataResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def report_data(
        self,
        msg_report_data: "MsgReportData",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgReportDataResponse":
        return await self._unary_unary(
            "/oracle.v1.Msg/ReportData",
            msg_report_data,
            MsgReportDataResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def create_data_source(
        self,
        msg_create_data_source: "MsgCreateDataSource",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgCreateDataSourceResponse":
        return await self._unary_unary(
            "/oracle.v1.Msg/CreateDataSource",
            msg_create_data_source,
            MsgCreateDataSourceResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def edit_data_source(
        self,
        msg_edit_data_source: "MsgEditDataSource",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgEditDataSourceResponse":
        return await self._unary_unary(
            "/oracle.v1.Msg/EditDataSource",
            msg_edit_data_source,
            MsgEditDataSourceResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def create_oracle_script(
        self,
        msg_create_oracle_script: "MsgCreateOracleScript",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgCreateOracleScriptResponse":
        return await self._unary_unary(
            "/oracle.v1.Msg/CreateOracleScript",
            msg_create_oracle_script,
            MsgCreateOracleScriptResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def edit_oracle_script(
        self,
        msg_edit_oracle_script: "MsgEditOracleScript",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgEditOracleScriptResponse":
        return await self._unary_unary(
            "/oracle.v1.Msg/EditOracleScript",
            msg_edit_oracle_script,
            MsgEditOracleScriptResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def activate(
        self,
        msg_activate: "MsgActivate",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgActivateResponse":
        return await self._unary_unary(
            "/oracle.v1.Msg/Activate",
            msg_activate,
            MsgActivateResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryStub(betterproto.ServiceStub):
    async def counts(
        self,
        query_counts_request: "QueryCountsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryCountsResponse":
        return await self._unary_unary(
            "/oracle.v1.Query/Counts",
            query_counts_request,
            QueryCountsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def data(
        self,
        query_data_request: "QueryDataRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryDataResponse":
        return await self._unary_unary(
            "/oracle.v1.Query/Data",
            query_data_request,
            QueryDataResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def data_source(
        self,
        query_data_source_request: "QueryDataSourceRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryDataSourceResponse":
        return await self._unary_unary(
            "/oracle.v1.Query/DataSource",
            query_data_source_request,
            QueryDataSourceResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def oracle_script(
        self,
        query_oracle_script_request: "QueryOracleScriptRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryOracleScriptResponse":
        return await self._unary_unary(
            "/oracle.v1.Query/OracleScript",
            query_oracle_script_request,
            QueryOracleScriptResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def request(
        self,
        query_request_request: "QueryRequestRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryRequestResponse":
        return await self._unary_unary(
            "/oracle.v1.Query/Request",
            query_request_request,
            QueryRequestResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def pending_requests(
        self,
        query_pending_requests_request: "QueryPendingRequestsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryPendingRequestsResponse":
        return await self._unary_unary(
            "/oracle.v1.Query/PendingRequests",
            query_pending_requests_request,
            QueryPendingRequestsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def validator(
        self,
        query_validator_request: "QueryValidatorRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryValidatorResponse":
        return await self._unary_unary(
            "/oracle.v1.Query/Validator",
            query_validator_request,
            QueryValidatorResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def is_reporter(
        self,
        query_is_reporter_request: "QueryIsReporterRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryIsReporterResponse":
        return await self._unary_unary(
            "/oracle.v1.Query/IsReporter",
            query_is_reporter_request,
            QueryIsReporterResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def reporters(
        self,
        query_reporters_request: "QueryReportersRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryReportersResponse":
        return await self._unary_unary(
            "/oracle.v1.Query/Reporters",
            query_reporters_request,
            QueryReportersResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def active_validators(
        self,
        query_active_validators_request: "QueryActiveValidatorsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryActiveValidatorsResponse":
        return await self._unary_unary(
            "/oracle.v1.Query/ActiveValidators",
            query_active_validators_request,
            QueryActiveValidatorsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def params(
        self,
        query_params_request: "QueryParamsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryParamsResponse":
        return await self._unary_unary(
            "/oracle.v1.Query/Params",
            query_params_request,
            QueryParamsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def request_search(
        self,
        query_request_search_request: "QueryRequestSearchRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryRequestSearchResponse":
        return await self._unary_unary(
            "/oracle.v1.Query/RequestSearch",
            query_request_search_request,
            QueryRequestSearchResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def request_price(
        self,
        query_request_price_request: "QueryRequestPriceRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryRequestPriceResponse":
        return await self._unary_unary(
            "/oracle.v1.Query/RequestPrice",
            query_request_price_request,
            QueryRequestPriceResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def request_verification(
        self,
        query_request_verification_request: "QueryRequestVerificationRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryRequestVerificationResponse":
        return await self._unary_unary(
            "/oracle.v1.Query/RequestVerification",
            query_request_verification_request,
            QueryRequestVerificationResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MsgBase(ServiceBase):
    async def request_data(self, msg_request_data: "MsgRequestData") -> "MsgRequestDataResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def report_data(self, msg_report_data: "MsgReportData") -> "MsgReportDataResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def create_data_source(self, msg_create_data_source: "MsgCreateDataSource") -> "MsgCreateDataSourceResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def edit_data_source(self, msg_edit_data_source: "MsgEditDataSource") -> "MsgEditDataSourceResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def create_oracle_script(
        self, msg_create_oracle_script: "MsgCreateOracleScript"
    ) -> "MsgCreateOracleScriptResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def edit_oracle_script(self, msg_edit_oracle_script: "MsgEditOracleScript") -> "MsgEditOracleScriptResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def activate(self, msg_activate: "MsgActivate") -> "MsgActivateResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_request_data(
        self, stream: "grpclib.server.Stream[MsgRequestData, MsgRequestDataResponse]"
    ) -> None:
        request = await stream.recv_message()
        response = await self.request_data(request)
        await stream.send_message(response)

    async def __rpc_report_data(self, stream: "grpclib.server.Stream[MsgReportData, MsgReportDataResponse]") -> None:
        request = await stream.recv_message()
        response = await self.report_data(request)
        await stream.send_message(response)

    async def __rpc_create_data_source(
        self,
        stream: "grpclib.server.Stream[MsgCreateDataSource, MsgCreateDataSourceResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.create_data_source(request)
        await stream.send_message(response)

    async def __rpc_edit_data_source(
        self,
        stream: "grpclib.server.Stream[MsgEditDataSource, MsgEditDataSourceResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.edit_data_source(request)
        await stream.send_message(response)

    async def __rpc_create_oracle_script(
        self,
        stream: "grpclib.server.Stream[MsgCreateOracleScript, MsgCreateOracleScriptResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.create_oracle_script(request)
        await stream.send_message(response)

    async def __rpc_edit_oracle_script(
        self,
        stream: "grpclib.server.Stream[MsgEditOracleScript, MsgEditOracleScriptResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.edit_oracle_script(request)
        await stream.send_message(response)

    async def __rpc_activate(self, stream: "grpclib.server.Stream[MsgActivate, MsgActivateResponse]") -> None:
        request = await stream.recv_message()
        response = await self.activate(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/oracle.v1.Msg/RequestData": grpclib.const.Handler(
                self.__rpc_request_data,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgRequestData,
                MsgRequestDataResponse,
            ),
            "/oracle.v1.Msg/ReportData": grpclib.const.Handler(
                self.__rpc_report_data,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgReportData,
                MsgReportDataResponse,
            ),
            "/oracle.v1.Msg/CreateDataSource": grpclib.const.Handler(
                self.__rpc_create_data_source,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgCreateDataSource,
                MsgCreateDataSourceResponse,
            ),
            "/oracle.v1.Msg/EditDataSource": grpclib.const.Handler(
                self.__rpc_edit_data_source,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgEditDataSource,
                MsgEditDataSourceResponse,
            ),
            "/oracle.v1.Msg/CreateOracleScript": grpclib.const.Handler(
                self.__rpc_create_oracle_script,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgCreateOracleScript,
                MsgCreateOracleScriptResponse,
            ),
            "/oracle.v1.Msg/EditOracleScript": grpclib.const.Handler(
                self.__rpc_edit_oracle_script,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgEditOracleScript,
                MsgEditOracleScriptResponse,
            ),
            "/oracle.v1.Msg/Activate": grpclib.const.Handler(
                self.__rpc_activate,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgActivate,
                MsgActivateResponse,
            ),
        }


class QueryBase(ServiceBase):
    async def counts(self, query_counts_request: "QueryCountsRequest") -> "QueryCountsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def data(self, query_data_request: "QueryDataRequest") -> "QueryDataResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def data_source(self, query_data_source_request: "QueryDataSourceRequest") -> "QueryDataSourceResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def oracle_script(
        self, query_oracle_script_request: "QueryOracleScriptRequest"
    ) -> "QueryOracleScriptResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def request(self, query_request_request: "QueryRequestRequest") -> "QueryRequestResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def pending_requests(
        self, query_pending_requests_request: "QueryPendingRequestsRequest"
    ) -> "QueryPendingRequestsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def validator(self, query_validator_request: "QueryValidatorRequest") -> "QueryValidatorResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def is_reporter(self, query_is_reporter_request: "QueryIsReporterRequest") -> "QueryIsReporterResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def reporters(self, query_reporters_request: "QueryReportersRequest") -> "QueryReportersResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def active_validators(
        self, query_active_validators_request: "QueryActiveValidatorsRequest"
    ) -> "QueryActiveValidatorsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def params(self, query_params_request: "QueryParamsRequest") -> "QueryParamsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def request_search(
        self, query_request_search_request: "QueryRequestSearchRequest"
    ) -> "QueryRequestSearchResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def request_price(
        self, query_request_price_request: "QueryRequestPriceRequest"
    ) -> "QueryRequestPriceResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def request_verification(
        self, query_request_verification_request: "QueryRequestVerificationRequest"
    ) -> "QueryRequestVerificationResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_counts(self, stream: "grpclib.server.Stream[QueryCountsRequest, QueryCountsResponse]") -> None:
        request = await stream.recv_message()
        response = await self.counts(request)
        await stream.send_message(response)

    async def __rpc_data(self, stream: "grpclib.server.Stream[QueryDataRequest, QueryDataResponse]") -> None:
        request = await stream.recv_message()
        response = await self.data(request)
        await stream.send_message(response)

    async def __rpc_data_source(
        self,
        stream: "grpclib.server.Stream[QueryDataSourceRequest, QueryDataSourceResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.data_source(request)
        await stream.send_message(response)

    async def __rpc_oracle_script(
        self,
        stream: "grpclib.server.Stream[QueryOracleScriptRequest, QueryOracleScriptResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.oracle_script(request)
        await stream.send_message(response)

    async def __rpc_request(self, stream: "grpclib.server.Stream[QueryRequestRequest, QueryRequestResponse]") -> None:
        request = await stream.recv_message()
        response = await self.request(request)
        await stream.send_message(response)

    async def __rpc_pending_requests(
        self,
        stream: "grpclib.server.Stream[QueryPendingRequestsRequest, QueryPendingRequestsResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.pending_requests(request)
        await stream.send_message(response)

    async def __rpc_validator(
        self,
        stream: "grpclib.server.Stream[QueryValidatorRequest, QueryValidatorResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.validator(request)
        await stream.send_message(response)

    async def __rpc_is_reporter(
        self,
        stream: "grpclib.server.Stream[QueryIsReporterRequest, QueryIsReporterResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.is_reporter(request)
        await stream.send_message(response)

    async def __rpc_reporters(
        self,
        stream: "grpclib.server.Stream[QueryReportersRequest, QueryReportersResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.reporters(request)
        await stream.send_message(response)

    async def __rpc_active_validators(
        self,
        stream: "grpclib.server.Stream[QueryActiveValidatorsRequest, QueryActiveValidatorsResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.active_validators(request)
        await stream.send_message(response)

    async def __rpc_params(self, stream: "grpclib.server.Stream[QueryParamsRequest, QueryParamsResponse]") -> None:
        request = await stream.recv_message()
        response = await self.params(request)
        await stream.send_message(response)

    async def __rpc_request_search(
        self,
        stream: "grpclib.server.Stream[QueryRequestSearchRequest, QueryRequestSearchResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.request_search(request)
        await stream.send_message(response)

    async def __rpc_request_price(
        self,
        stream: "grpclib.server.Stream[QueryRequestPriceRequest, QueryRequestPriceResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.request_price(request)
        await stream.send_message(response)

    async def __rpc_request_verification(
        self,
        stream: "grpclib.server.Stream[QueryRequestVerificationRequest, QueryRequestVerificationResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.request_verification(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/oracle.v1.Query/Counts": grpclib.const.Handler(
                self.__rpc_counts,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryCountsRequest,
                QueryCountsResponse,
            ),
            "/oracle.v1.Query/Data": grpclib.const.Handler(
                self.__rpc_data,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryDataRequest,
                QueryDataResponse,
            ),
            "/oracle.v1.Query/DataSource": grpclib.const.Handler(
                self.__rpc_data_source,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryDataSourceRequest,
                QueryDataSourceResponse,
            ),
            "/oracle.v1.Query/OracleScript": grpclib.const.Handler(
                self.__rpc_oracle_script,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryOracleScriptRequest,
                QueryOracleScriptResponse,
            ),
            "/oracle.v1.Query/Request": grpclib.const.Handler(
                self.__rpc_request,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryRequestRequest,
                QueryRequestResponse,
            ),
            "/oracle.v1.Query/PendingRequests": grpclib.const.Handler(
                self.__rpc_pending_requests,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryPendingRequestsRequest,
                QueryPendingRequestsResponse,
            ),
            "/oracle.v1.Query/Validator": grpclib.const.Handler(
                self.__rpc_validator,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryValidatorRequest,
                QueryValidatorResponse,
            ),
            "/oracle.v1.Query/IsReporter": grpclib.const.Handler(
                self.__rpc_is_reporter,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryIsReporterRequest,
                QueryIsReporterResponse,
            ),
            "/oracle.v1.Query/Reporters": grpclib.const.Handler(
                self.__rpc_reporters,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryReportersRequest,
                QueryReportersResponse,
            ),
            "/oracle.v1.Query/ActiveValidators": grpclib.const.Handler(
                self.__rpc_active_validators,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryActiveValidatorsRequest,
                QueryActiveValidatorsResponse,
            ),
            "/oracle.v1.Query/Params": grpclib.const.Handler(
                self.__rpc_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryParamsRequest,
                QueryParamsResponse,
            ),
            "/oracle.v1.Query/RequestSearch": grpclib.const.Handler(
                self.__rpc_request_search,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryRequestSearchRequest,
                QueryRequestSearchResponse,
            ),
            "/oracle.v1.Query/RequestPrice": grpclib.const.Handler(
                self.__rpc_request_price,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryRequestPriceRequest,
                QueryRequestPriceResponse,
            ),
            "/oracle.v1.Query/RequestVerification": grpclib.const.Handler(
                self.__rpc_request_verification,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryRequestVerificationRequest,
                QueryRequestVerificationResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: tendermint/libs/bits/types.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import betterproto


@dataclass(eq=False, repr=False)
class BitArray(betterproto.Message):
    bits: int = betterproto.int64_field(1)
    elems: List[int] = betterproto.uint64_field(2)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/feegrant/v1beta1/feegrant.proto, cosmos/feegrant/v1beta1/genesis.proto, cosmos/feegrant/v1beta1/query.proto, cosmos/feegrant/v1beta1/tx.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import (
    datetime,
    timedelta,
)
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ...base import v1beta1 as __base_v1_beta1__
from ...base.query import v1beta1 as __base_query_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class MsgGrantAllowance(betterproto.Message):
    """
    MsgGrantAllowance adds permission for Grantee to spend up to Allowance of
    fees from the account of Granter.
    """

    granter: str = betterproto.string_field(1)
    """
    granter is the address of the user granting an allowance of their funds.
    """

    grantee: str = betterproto.string_field(2)
    """
    grantee is the address of the user being granted an allowance of another
    user's funds.
    """

    allowance: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(3)
    """allowance can be any of basic and filtered fee allowance."""


@dataclass(eq=False, repr=False)
class MsgGrantAllowanceResponse(betterproto.Message):
    """
    MsgGrantAllowanceResponse defines the Msg/GrantAllowanceResponse response
    type.
    """

    pass


@dataclass(eq=False, repr=False)
class MsgRevokeAllowance(betterproto.Message):
    """
    MsgRevokeAllowance removes any existing Allowance from Granter to Grantee.
    """

    granter: str = betterproto.string_field(1)
    """
    granter is the address of the user granting an allowance of their funds.
    """

    grantee: str = betterproto.string_field(2)
    """
    grantee is the address of the user being granted an allowance of another
    user's funds.
    """


@dataclass(eq=False, repr=False)
class MsgRevokeAllowanceResponse(betterproto.Message):
    """
    MsgRevokeAllowanceResponse defines the Msg/RevokeAllowanceResponse response
    type.
    """

    pass


@dataclass(eq=False, repr=False)
class BasicAllowance(betterproto.Message):
    """
    BasicAllowance implements Allowance with a one-time grant of tokens that
    optionally expires. The grantee can use up to SpendLimit to cover fees.
    """

    spend_limit: List["__base_v1_beta1__.Coin"] = betterproto.message_field(1)
    """
    spend_limit specifies the maximum amount of tokens that can be spent by
    this allowance and will be updated as tokens are spent. If it is empty,
    there is no spend limit and any amount of coins can be spent.
    """

    expiration: datetime = betterproto.message_field(2)
    """expiration specifies an optional time when this allowance expires"""


@dataclass(eq=False, repr=False)
class PeriodicAllowance(betterproto.Message):
    """
    PeriodicAllowance extends Allowance to allow for both a maximum cap, as
    well as a limit per time period.
    """

    basic: "BasicAllowance" = betterproto.message_field(1)
    """basic specifies a struct of `BasicAllowance`"""

    period: timedelta = betterproto.message_field(2)
    """
    period specifies the time duration in which period_spend_limit coins can be
    spent before that allowance is reset
    """

    period_spend_limit: List["__base_v1_beta1__.Coin"] = betterproto.message_field(3)
    """
    period_spend_limit specifies the maximum number of coins that can be spent
    in the period
    """

    period_can_spend: List["__base_v1_beta1__.Coin"] = betterproto.message_field(4)
    """
    period_can_spend is the number of coins left to be spent before the
    period_reset time
    """

    period_reset: datetime = betterproto.message_field(5)
    """
    period_reset is the time at which this period resets and a new one begins,
    it is calculated from the start time of the first transaction after the
    last period ended
    """


@dataclass(eq=False, repr=False)
class AllowedMsgAllowance(betterproto.Message):
    """
    AllowedMsgAllowance creates allowance only for specified message types.
    """

    allowance: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(1)
    """allowance can be any of basic and filtered fee allowance."""

    allowed_messages: List[str] = betterproto.string_field(2)
    """
    allowed_messages are the messages for which the grantee has the access.
    """


@dataclass(eq=False, repr=False)
class Grant(betterproto.Message):
    """Grant is stored in the KVStore to record a grant with full context"""

    granter: str = betterproto.string_field(1)
    """
    granter is the address of the user granting an allowance of their funds.
    """

    grantee: str = betterproto.string_field(2)
    """
    grantee is the address of the user being granted an allowance of another
    user's funds.
    """

    allowance: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(3)
    """allowance can be any of basic and filtered fee allowance."""


@dataclass(eq=False, repr=False)
class QueryAllowanceRequest(betterproto.Message):
    """
    QueryAllowanceRequest is the request type for the Query/Allowance RPC
    method.
    """

    granter: str = betterproto.string_field(1)
    """
    granter is the address of the user granting an allowance of their funds.
    """

    grantee: str = betterproto.string_field(2)
    """
    grantee is the address of the user being granted an allowance of another
    user's funds.
    """


@dataclass(eq=False, repr=False)
class QueryAllowanceResponse(betterproto.Message):
    """
    QueryAllowanceResponse is the response type for the Query/Allowance RPC
    method.
    """

    allowance: "Grant" = betterproto.message_field(1)
    """allowance is a allowance granted for grantee by granter."""


@dataclass(eq=False, repr=False)
class QueryAllowancesRequest(betterproto.Message):
    """
    QueryAllowancesRequest is the request type for the Query/Allowances RPC
    method.
    """

    grantee: str = betterproto.string_field(1)
    pagination: "__base_query_v1_beta1__.PageRequest" = betterproto.message_field(2)
    """pagination defines an pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryAllowancesResponse(betterproto.Message):
    """
    QueryAllowancesResponse is the response type for the Query/Allowances RPC
    method.
    """

    allowances: List["Grant"] = betterproto.message_field(1)
    """allowances are allowance's granted for grantee by granter."""

    pagination: "__base_query_v1_beta1__.PageResponse" = betterproto.message_field(2)
    """pagination defines an pagination for the response."""


@dataclass(eq=False, repr=False)
class QueryAllowancesByGranterRequest(betterproto.Message):
    """
    QueryAllowancesByGranterRequest is the request type for the
    Query/AllowancesByGranter RPC method.
    """

    granter: str = betterproto.string_field(1)
    pagination: "__base_query_v1_beta1__.PageRequest" = betterproto.message_field(2)
    """pagination defines an pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryAllowancesByGranterResponse(betterproto.Message):
    """
    QueryAllowancesByGranterResponse is the response type for the
    Query/AllowancesByGranter RPC method.
    """

    allowances: List["Grant"] = betterproto.message_field(1)
    """allowances that have been issued by the granter."""

    pagination: "__base_query_v1_beta1__.PageResponse" = betterproto.message_field(2)
    """pagination defines an pagination for the response."""


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """
    GenesisState contains a set of fee allowances, persisted from the store
    """

    allowances: List["Grant"] = betterproto.message_field(1)


class MsgStub(betterproto.ServiceStub):
    async def grant_allowance(
        self,
        msg_grant_allowance: "MsgGrantAllowance",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgGrantAllowanceResponse":
        return await self._unary_unary(
            "/cosmos.feegrant.v1beta1.Msg/GrantAllowance",
            msg_grant_allowance,
            MsgGrantAllowanceResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def revoke_allowance(
        self,
        msg_revoke_allowance: "MsgRevokeAllowance",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgRevokeAllowanceResponse":
        return await self._unary_unary(
            "/cosmos.feegrant.v1beta1.Msg/RevokeAllowance",
            msg_revoke_allowance,
            MsgRevokeAllowanceResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryStub(betterproto.ServiceStub):
    async def allowance(
        self,
        query_allowance_request: "QueryAllowanceRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryAllowanceResponse":
        return await self._unary_unary(
            "/cosmos.feegrant.v1beta1.Query/Allowance",
            query_allowance_request,
            QueryAllowanceResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def allowances(
        self,
        query_allowances_request: "QueryAllowancesRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryAllowancesResponse":
        return await self._unary_unary(
            "/cosmos.feegrant.v1beta1.Query/Allowances",
            query_allowances_request,
            QueryAllowancesResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def allowances_by_granter(
        self,
        query_allowances_by_granter_request: "QueryAllowancesByGranterRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryAllowancesByGranterResponse":
        return await self._unary_unary(
            "/cosmos.feegrant.v1beta1.Query/AllowancesByGranter",
            query_allowances_by_granter_request,
            QueryAllowancesByGranterResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MsgBase(ServiceBase):
    async def grant_allowance(self, msg_grant_allowance: "MsgGrantAllowance") -> "MsgGrantAllowanceResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def revoke_allowance(self, msg_revoke_allowance: "MsgRevokeAllowance") -> "MsgRevokeAllowanceResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_grant_allowance(
        self,
        stream: "grpclib.server.Stream[MsgGrantAllowance, MsgGrantAllowanceResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.grant_allowance(request)
        await stream.send_message(response)

    async def __rpc_revoke_allowance(
        self,
        stream: "grpclib.server.Stream[MsgRevokeAllowance, MsgRevokeAllowanceResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.revoke_allowance(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.feegrant.v1beta1.Msg/GrantAllowance": grpclib.const.Handler(
                self.__rpc_grant_allowance,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgGrantAllowance,
                MsgGrantAllowanceResponse,
            ),
            "/cosmos.feegrant.v1beta1.Msg/RevokeAllowance": grpclib.const.Handler(
                self.__rpc_revoke_allowance,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgRevokeAllowance,
                MsgRevokeAllowanceResponse,
            ),
        }


class QueryBase(ServiceBase):
    async def allowance(self, query_allowance_request: "QueryAllowanceRequest") -> "QueryAllowanceResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def allowances(self, query_allowances_request: "QueryAllowancesRequest") -> "QueryAllowancesResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def allowances_by_granter(
        self, query_allowances_by_granter_request: "QueryAllowancesByGranterRequest"
    ) -> "QueryAllowancesByGranterResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_allowance(
        self,
        stream: "grpclib.server.Stream[QueryAllowanceRequest, QueryAllowanceResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.allowance(request)
        await stream.send_message(response)

    async def __rpc_allowances(
        self,
        stream: "grpclib.server.Stream[QueryAllowancesRequest, QueryAllowancesResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.allowances(request)
        await stream.send_message(response)

    async def __rpc_allowances_by_granter(
        self,
        stream: "grpclib.server.Stream[QueryAllowancesByGranterRequest, QueryAllowancesByGranterResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.allowances_by_granter(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.feegrant.v1beta1.Query/Allowance": grpclib.const.Handler(
                self.__rpc_allowance,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryAllowanceRequest,
                QueryAllowanceResponse,
            ),
            "/cosmos.feegrant.v1beta1.Query/Allowances": grpclib.const.Handler(
                self.__rpc_allowances,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryAllowancesRequest,
                QueryAllowancesResponse,
            ),
            "/cosmos.feegrant.v1beta1.Query/AllowancesByGranter": grpclib.const.Handler(
                self.__rpc_allowances_by_granter,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryAllowancesByGranterRequest,
                QueryAllowancesByGranterResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/authz/v1beta1/authz.proto, cosmos/authz/v1beta1/event.proto, cosmos/authz/v1beta1/genesis.proto, cosmos/authz/v1beta1/query.proto, cosmos/authz/v1beta1/tx.proto
# plugin: python-betterproto
from dataclasses import dataclass
from datetime import datetime
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ...base.query import v1beta1 as __base_query_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class GenericAuthorization(betterproto.Message):
    """
    GenericAuthorization gives the grantee unrestricted permissions to execute
    the provided method on behalf of the granter's account.
    """

    msg: str = betterproto.string_field(1)
    """
    Msg, identified by it's type URL, to grant unrestricted permissions to
    execute
    """


@dataclass(eq=False, repr=False)
class Grant(betterproto.Message):
    """
    Grant gives permissions to execute the provide method with expiration time.
    """

    authorization: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(1)
    expiration: datetime = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class GrantAuthorization(betterproto.Message):
    """
    GrantAuthorization extends a grant with both the addresses of the grantee
    and granter. It is used in genesis.proto and query.proto Since: cosmos-sdk
    0.45.2
    """

    granter: str = betterproto.string_field(1)
    grantee: str = betterproto.string_field(2)
    authorization: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(3)
    expiration: datetime = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class MsgGrant(betterproto.Message):
    """
    MsgGrant is a request type for Grant method. It declares authorization to
    the grantee on behalf of the granter with the provided expiration time.
    """

    granter: str = betterproto.string_field(1)
    grantee: str = betterproto.string_field(2)
    grant: "Grant" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class MsgExecResponse(betterproto.Message):
    """MsgExecResponse defines the Msg/MsgExecResponse response type."""

    results: List[bytes] = betterproto.bytes_field(1)


@dataclass(eq=False, repr=False)
class MsgExec(betterproto.Message):
    """
    MsgExec attempts to execute the provided messages using authorizations
    granted to the grantee. Each message should have only one signer
    corresponding to the granter of the authorization.
    """

    grantee: str = betterproto.string_field(1)
    msgs: List["betterproto_lib_google_protobuf.Any"] = betterproto.message_field(2)
    """
    Authorization Msg requests to execute. Each msg must implement
    Authorization interface The x/authz will try to find a grant matching
    (msg.signers[0], grantee, MsgTypeURL(msg)) triple and validate it.
    """


@dataclass(eq=False, repr=False)
class MsgGrantResponse(betterproto.Message):
    """MsgGrantResponse defines the Msg/MsgGrant response type."""

    pass


@dataclass(eq=False, repr=False)
class MsgRevoke(betterproto.Message):
    """
    MsgRevoke revokes any authorization with the provided sdk.Msg type on the
    granter's account with that has been granted to the grantee.
    """

    granter: str = betterproto.string_field(1)
    grantee: str = betterproto.string_field(2)
    msg_type_url: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class MsgRevokeResponse(betterproto.Message):
    """MsgRevokeResponse defines the Msg/MsgRevokeResponse response type."""

    pass


@dataclass(eq=False, repr=False)
class QueryGrantsRequest(betterproto.Message):
    """
    QueryGrantsRequest is the request type for the Query/Grants RPC method.
    """

    granter: str = betterproto.string_field(1)
    grantee: str = betterproto.string_field(2)
    msg_type_url: str = betterproto.string_field(3)
    """
    Optional, msg_type_url, when set, will query only grants matching given msg
    type.
    """

    pagination: "__base_query_v1_beta1__.PageRequest" = betterproto.message_field(4)
    """pagination defines an pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryGrantsResponse(betterproto.Message):
    """
    QueryGrantsResponse is the response type for the Query/Authorizations RPC
    method.
    """

    grants: List["Grant"] = betterproto.message_field(1)
    """authorizations is a list of grants granted for grantee by granter."""

    pagination: "__base_query_v1_beta1__.PageResponse" = betterproto.message_field(2)
    """pagination defines an pagination for the response."""


@dataclass(eq=False, repr=False)
class QueryGranterGrantsRequest(betterproto.Message):
    """
    QueryGranterGrantsRequest is the request type for the Query/GranterGrants
    RPC method.
    """

    granter: str = betterproto.string_field(1)
    pagination: "__base_query_v1_beta1__.PageRequest" = betterproto.message_field(2)
    """pagination defines an pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryGranterGrantsResponse(betterproto.Message):
    """
    QueryGranterGrantsResponse is the response type for the Query/GranterGrants
    RPC method.
    """

    grants: List["GrantAuthorization"] = betterproto.message_field(1)
    """grants is a list of grants granted by the granter."""

    pagination: "__base_query_v1_beta1__.PageResponse" = betterproto.message_field(2)
    """pagination defines an pagination for the response."""


@dataclass(eq=False, repr=False)
class QueryGranteeGrantsRequest(betterproto.Message):
    """
    QueryGranteeGrantsRequest is the request type for the Query/IssuedGrants
    RPC method.
    """

    grantee: str = betterproto.string_field(1)
    pagination: "__base_query_v1_beta1__.PageRequest" = betterproto.message_field(2)
    """pagination defines an pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryGranteeGrantsResponse(betterproto.Message):
    """
    QueryGranteeGrantsResponse is the response type for the Query/GranteeGrants
    RPC method.
    """

    grants: List["GrantAuthorization"] = betterproto.message_field(1)
    """grants is a list of grants granted to the grantee."""

    pagination: "__base_query_v1_beta1__.PageResponse" = betterproto.message_field(2)
    """pagination defines an pagination for the response."""


@dataclass(eq=False, repr=False)
class EventGrant(betterproto.Message):
    """EventGrant is emitted on Msg/Grant"""

    msg_type_url: str = betterproto.string_field(2)
    """Msg type URL for which an autorization is granted"""

    granter: str = betterproto.string_field(3)
    """Granter account address"""

    grantee: str = betterproto.string_field(4)
    """Grantee account address"""


@dataclass(eq=False, repr=False)
class EventRevoke(betterproto.Message):
    """EventRevoke is emitted on Msg/Revoke"""

    msg_type_url: str = betterproto.string_field(2)
    """Msg type URL for which an autorization is revoked"""

    granter: str = betterproto.string_field(3)
    """Granter account address"""

    grantee: str = betterproto.string_field(4)
    """Grantee account address"""


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the authz module's genesis state."""

    authorization: List["GrantAuthorization"] = betterproto.message_field(1)


class MsgStub(betterproto.ServiceStub):
    async def grant(
        self,
        msg_grant: "MsgGrant",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgGrantResponse":
        return await self._unary_unary(
            "/cosmos.authz.v1beta1.Msg/Grant",
            msg_grant,
            MsgGrantResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def exec(
        self,
        msg_exec: "MsgExec",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgExecResponse":
        return await self._unary_unary(
            "/cosmos.authz.v1beta1.Msg/Exec",
            msg_exec,
            MsgExecResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def revoke(
        self,
        msg_revoke: "MsgRevoke",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgRevokeResponse":
        return await self._unary_unary(
            "/cosmos.authz.v1beta1.Msg/Revoke",
            msg_revoke,
            MsgRevokeResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryStub(betterproto.ServiceStub):
    async def grants(
        self,
        query_grants_request: "QueryGrantsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryGrantsResponse":
        return await self._unary_unary(
            "/cosmos.authz.v1beta1.Query/Grants",
            query_grants_request,
            QueryGrantsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def granter_grants(
        self,
        query_granter_grants_request: "QueryGranterGrantsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryGranterGrantsResponse":
        return await self._unary_unary(
            "/cosmos.authz.v1beta1.Query/GranterGrants",
            query_granter_grants_request,
            QueryGranterGrantsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def grantee_grants(
        self,
        query_grantee_grants_request: "QueryGranteeGrantsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryGranteeGrantsResponse":
        return await self._unary_unary(
            "/cosmos.authz.v1beta1.Query/GranteeGrants",
            query_grantee_grants_request,
            QueryGranteeGrantsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MsgBase(ServiceBase):
    async def grant(self, msg_grant: "MsgGrant") -> "MsgGrantResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def exec(self, msg_exec: "MsgExec") -> "MsgExecResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def revoke(self, msg_revoke: "MsgRevoke") -> "MsgRevokeResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_grant(self, stream: "grpclib.server.Stream[MsgGrant, MsgGrantResponse]") -> None:
        request = await stream.recv_message()
        response = await self.grant(request)
        await stream.send_message(response)

    async def __rpc_exec(self, stream: "grpclib.server.Stream[MsgExec, MsgExecResponse]") -> None:
        request = await stream.recv_message()
        response = await self.exec(request)
        await stream.send_message(response)

    async def __rpc_revoke(self, stream: "grpclib.server.Stream[MsgRevoke, MsgRevokeResponse]") -> None:
        request = await stream.recv_message()
        response = await self.revoke(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.authz.v1beta1.Msg/Grant": grpclib.const.Handler(
                self.__rpc_grant,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgGrant,
                MsgGrantResponse,
            ),
            "/cosmos.authz.v1beta1.Msg/Exec": grpclib.const.Handler(
                self.__rpc_exec,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgExec,
                MsgExecResponse,
            ),
            "/cosmos.authz.v1beta1.Msg/Revoke": grpclib.const.Handler(
                self.__rpc_revoke,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgRevoke,
                MsgRevokeResponse,
            ),
        }


class QueryBase(ServiceBase):
    async def grants(self, query_grants_request: "QueryGrantsRequest") -> "QueryGrantsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def granter_grants(
        self, query_granter_grants_request: "QueryGranterGrantsRequest"
    ) -> "QueryGranterGrantsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def grantee_grants(
        self, query_grantee_grants_request: "QueryGranteeGrantsRequest"
    ) -> "QueryGranteeGrantsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_grants(self, stream: "grpclib.server.Stream[QueryGrantsRequest, QueryGrantsResponse]") -> None:
        request = await stream.recv_message()
        response = await self.grants(request)
        await stream.send_message(response)

    async def __rpc_granter_grants(
        self,
        stream: "grpclib.server.Stream[QueryGranterGrantsRequest, QueryGranterGrantsResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.granter_grants(request)
        await stream.send_message(response)

    async def __rpc_grantee_grants(
        self,
        stream: "grpclib.server.Stream[QueryGranteeGrantsRequest, QueryGranteeGrantsResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.grantee_grants(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.authz.v1beta1.Query/Grants": grpclib.const.Handler(
                self.__rpc_grants,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryGrantsRequest,
                QueryGrantsResponse,
            ),
            "/cosmos.authz.v1beta1.Query/GranterGrants": grpclib.const.Handler(
                self.__rpc_granter_grants,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryGranterGrantsRequest,
                QueryGranterGrantsResponse,
            ),
            "/cosmos.authz.v1beta1.Query/GranteeGrants": grpclib.const.Handler(
                self.__rpc_grantee_grants,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryGranteeGrantsRequest,
                QueryGranteeGrantsResponse,
            ),
        }



BECH32_ADDR_ACC_PREFIX = "band"
BECH32_ADDR_VAL_PREFIX = "bandvaloper"
BECH32_ADDR_CONS_PREFIX = "bandvalcons"

BECH32_PUBKEY_ACC_PREFIX = "bandpub"
BECH32_PUBKEY_VAL_PREFIX = "bandvaloperpub"
BECH32_PUBKEY_CONS_PREFIX = "bandvalconspub"

DEFAULT_DERIVATION_PATH = "m/44'/494'/0'/0/0"
DEFAULT_LEDGER_DERIVATION_PATH = "m/44'/118'/0'/0/0"



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/upgrade/v1beta1/query.proto, cosmos/upgrade/v1beta1/upgrade.proto
# plugin: python-betterproto
import warnings
from dataclasses import dataclass
from datetime import datetime
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class Plan(betterproto.Message):
    """
    Plan specifies information about a planned upgrade and when it should
    occur.
    """

    name: str = betterproto.string_field(1)
    """
    Sets the name for the upgrade. This name will be used by the upgraded
    version of the software to apply any special "on-upgrade" commands during
    the first BeginBlock method after the upgrade is applied. It is also used
    to detect whether a software version can handle a given upgrade. If no
    upgrade handler with this name has been set in the software, it will be
    assumed that the software is out-of-date when the upgrade Time or Height is
    reached and the software will exit.
    """

    time: datetime = betterproto.message_field(2)
    """
    Deprecated: Time based upgrades have been deprecated. Time based upgrade
    logic has been removed from the SDK. If this field is not empty, an error
    will be thrown.
    """

    height: int = betterproto.int64_field(3)
    """
    The height at which the upgrade must be performed. Only used if Time is not
    set.
    """

    info: str = betterproto.string_field(4)
    """
    Any application specific upgrade info to be included on-chain such as a git
    commit that validators could automatically upgrade to
    """

    upgraded_client_state: "betterproto_lib_google_protobuf.Any" = betterproto.message_field(5)
    """
    Deprecated: UpgradedClientState field has been deprecated. IBC upgrade
    logic has been moved to the IBC module in the sub module 02-client. If this
    field is not empty, an error will be thrown.
    """

    def __post_init__(self) -> None:
        super().__post_init__()
        if self.is_set("time"):
            warnings.warn("Plan.time is deprecated", DeprecationWarning)
        if self.is_set("upgraded_client_state"):
            warnings.warn("Plan.upgraded_client_state is deprecated", DeprecationWarning)


@dataclass(eq=False, repr=False)
class SoftwareUpgradeProposal(betterproto.Message):
    """
    SoftwareUpgradeProposal is a gov Content type for initiating a software
    upgrade.
    """

    title: str = betterproto.string_field(1)
    description: str = betterproto.string_field(2)
    plan: "Plan" = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class CancelSoftwareUpgradeProposal(betterproto.Message):
    """
    CancelSoftwareUpgradeProposal is a gov Content type for cancelling a
    software upgrade.
    """

    title: str = betterproto.string_field(1)
    description: str = betterproto.string_field(2)


@dataclass(eq=False, repr=False)
class ModuleVersion(betterproto.Message):
    """
    ModuleVersion specifies a module and its consensus version. Since: cosmos-
    sdk 0.43
    """

    name: str = betterproto.string_field(1)
    """name of the app module"""

    version: int = betterproto.uint64_field(2)
    """consensus version of the app module"""


@dataclass(eq=False, repr=False)
class QueryCurrentPlanRequest(betterproto.Message):
    """
    QueryCurrentPlanRequest is the request type for the Query/CurrentPlan RPC
    method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryCurrentPlanResponse(betterproto.Message):
    """
    QueryCurrentPlanResponse is the response type for the Query/CurrentPlan RPC
    method.
    """

    plan: "Plan" = betterproto.message_field(1)
    """plan is the current upgrade plan."""


@dataclass(eq=False, repr=False)
class QueryAppliedPlanRequest(betterproto.Message):
    """
    QueryCurrentPlanRequest is the request type for the Query/AppliedPlan RPC
    method.
    """

    name: str = betterproto.string_field(1)
    """name is the name of the applied plan to query for."""


@dataclass(eq=False, repr=False)
class QueryAppliedPlanResponse(betterproto.Message):
    """
    QueryAppliedPlanResponse is the response type for the Query/AppliedPlan RPC
    method.
    """

    height: int = betterproto.int64_field(1)
    """height is the block height at which the plan was applied."""


@dataclass(eq=False, repr=False)
class QueryUpgradedConsensusStateRequest(betterproto.Message):
    """
    QueryUpgradedConsensusStateRequest is the request type for the
    Query/UpgradedConsensusState RPC method.
    """

    last_height: int = betterproto.int64_field(1)
    """
    last height of the current chain must be sent in request as this is the
    height under which next consensus state is stored
    """

    def __post_init__(self) -> None:
        warnings.warn("QueryUpgradedConsensusStateRequest is deprecated", DeprecationWarning)
        super().__post_init__()


@dataclass(eq=False, repr=False)
class QueryUpgradedConsensusStateResponse(betterproto.Message):
    """
    QueryUpgradedConsensusStateResponse is the response type for the
    Query/UpgradedConsensusState RPC method.
    """

    upgraded_consensus_state: bytes = betterproto.bytes_field(2)
    """Since: cosmos-sdk 0.43"""

    def __post_init__(self) -> None:
        warnings.warn("QueryUpgradedConsensusStateResponse is deprecated", DeprecationWarning)
        super().__post_init__()


@dataclass(eq=False, repr=False)
class QueryModuleVersionsRequest(betterproto.Message):
    """
    QueryModuleVersionsRequest is the request type for the Query/ModuleVersions
    RPC method. Since: cosmos-sdk 0.43
    """

    module_name: str = betterproto.string_field(1)
    """
    module_name is a field to query a specific module consensus version from
    state. Leaving this empty will fetch the full list of module versions from
    state
    """


@dataclass(eq=False, repr=False)
class QueryModuleVersionsResponse(betterproto.Message):
    """
    QueryModuleVersionsResponse is the response type for the
    Query/ModuleVersions RPC method. Since: cosmos-sdk 0.43
    """

    module_versions: List["ModuleVersion"] = betterproto.message_field(1)
    """
    module_versions is a list of module names with their consensus versions.
    """


class QueryStub(betterproto.ServiceStub):
    async def current_plan(
        self,
        query_current_plan_request: "QueryCurrentPlanRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryCurrentPlanResponse":
        return await self._unary_unary(
            "/cosmos.upgrade.v1beta1.Query/CurrentPlan",
            query_current_plan_request,
            QueryCurrentPlanResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def applied_plan(
        self,
        query_applied_plan_request: "QueryAppliedPlanRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryAppliedPlanResponse":
        return await self._unary_unary(
            "/cosmos.upgrade.v1beta1.Query/AppliedPlan",
            query_applied_plan_request,
            QueryAppliedPlanResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def upgraded_consensus_state(
        self,
        query_upgraded_consensus_state_request: "QueryUpgradedConsensusStateRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryUpgradedConsensusStateResponse":
        return await self._unary_unary(
            "/cosmos.upgrade.v1beta1.Query/UpgradedConsensusState",
            query_upgraded_consensus_state_request,
            QueryUpgradedConsensusStateResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def module_versions(
        self,
        query_module_versions_request: "QueryModuleVersionsRequest",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "QueryModuleVersionsResponse":
        return await self._unary_unary(
            "/cosmos.upgrade.v1beta1.Query/ModuleVersions",
            query_module_versions_request,
            QueryModuleVersionsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryBase(ServiceBase):
    async def current_plan(self, query_current_plan_request: "QueryCurrentPlanRequest") -> "QueryCurrentPlanResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def applied_plan(self, query_applied_plan_request: "QueryAppliedPlanRequest") -> "QueryAppliedPlanResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def upgraded_consensus_state(
        self,
        query_upgraded_consensus_state_request: "QueryUpgradedConsensusStateRequest",
    ) -> "QueryUpgradedConsensusStateResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def module_versions(
        self, query_module_versions_request: "QueryModuleVersionsRequest"
    ) -> "QueryModuleVersionsResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_current_plan(
        self,
        stream: "grpclib.server.Stream[QueryCurrentPlanRequest, QueryCurrentPlanResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.current_plan(request)
        await stream.send_message(response)

    async def __rpc_applied_plan(
        self,
        stream: "grpclib.server.Stream[QueryAppliedPlanRequest, QueryAppliedPlanResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.applied_plan(request)
        await stream.send_message(response)

    async def __rpc_upgraded_consensus_state(
        self,
        stream: "grpclib.server.Stream[QueryUpgradedConsensusStateRequest, QueryUpgradedConsensusStateResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.upgraded_consensus_state(request)
        await stream.send_message(response)

    async def __rpc_module_versions(
        self,
        stream: "grpclib.server.Stream[QueryModuleVersionsRequest, QueryModuleVersionsResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.module_versions(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.upgrade.v1beta1.Query/CurrentPlan": grpclib.const.Handler(
                self.__rpc_current_plan,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryCurrentPlanRequest,
                QueryCurrentPlanResponse,
            ),
            "/cosmos.upgrade.v1beta1.Query/AppliedPlan": grpclib.const.Handler(
                self.__rpc_applied_plan,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryAppliedPlanRequest,
                QueryAppliedPlanResponse,
            ),
            "/cosmos.upgrade.v1beta1.Query/UpgradedConsensusState": grpclib.const.Handler(
                self.__rpc_upgraded_consensus_state,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryUpgradedConsensusStateRequest,
                QueryUpgradedConsensusStateResponse,
            ),
            "/cosmos.upgrade.v1beta1.Query/ModuleVersions": grpclib.const.Handler(
                self.__rpc_module_versions,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryModuleVersionsRequest,
                QueryModuleVersionsResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/vesting/v1beta1/tx.proto, cosmos/vesting/v1beta1/vesting.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ...auth import v1beta1 as __auth_v1_beta1__
from ...base import v1beta1 as __base_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class MsgCreateVestingAccount(betterproto.Message):
    """
    MsgCreateVestingAccount defines a message that enables creating a vesting
    account.
    """

    from_address: str = betterproto.string_field(1)
    to_address: str = betterproto.string_field(2)
    amount: List["__base_v1_beta1__.Coin"] = betterproto.message_field(3)
    end_time: int = betterproto.int64_field(4)
    delayed: bool = betterproto.bool_field(5)


@dataclass(eq=False, repr=False)
class MsgCreateVestingAccountResponse(betterproto.Message):
    """
    MsgCreateVestingAccountResponse defines the Msg/CreateVestingAccount
    response type.
    """

    pass


@dataclass(eq=False, repr=False)
class BaseVestingAccount(betterproto.Message):
    """
    BaseVestingAccount implements the VestingAccount interface. It contains all
    the necessary fields needed for any vesting account implementation.
    """

    base_account: "__auth_v1_beta1__.BaseAccount" = betterproto.message_field(1)
    original_vesting: List["__base_v1_beta1__.Coin"] = betterproto.message_field(2)
    delegated_free: List["__base_v1_beta1__.Coin"] = betterproto.message_field(3)
    delegated_vesting: List["__base_v1_beta1__.Coin"] = betterproto.message_field(4)
    end_time: int = betterproto.int64_field(5)


@dataclass(eq=False, repr=False)
class ContinuousVestingAccount(betterproto.Message):
    """
    ContinuousVestingAccount implements the VestingAccount interface. It
    continuously vests by unlocking coins linearly with respect to time.
    """

    base_vesting_account: "BaseVestingAccount" = betterproto.message_field(1)
    start_time: int = betterproto.int64_field(2)


@dataclass(eq=False, repr=False)
class DelayedVestingAccount(betterproto.Message):
    """
    DelayedVestingAccount implements the VestingAccount interface. It vests all
    coins after a specific time, but non prior. In other words, it keeps them
    locked until a specified time.
    """

    base_vesting_account: "BaseVestingAccount" = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class Period(betterproto.Message):
    """Period defines a length of time and amount of coins that will vest."""

    length: int = betterproto.int64_field(1)
    amount: List["__base_v1_beta1__.Coin"] = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class PeriodicVestingAccount(betterproto.Message):
    """
    PeriodicVestingAccount implements the VestingAccount interface. It
    periodically vests by unlocking coins during each specified period.
    """

    base_vesting_account: "BaseVestingAccount" = betterproto.message_field(1)
    start_time: int = betterproto.int64_field(2)
    vesting_periods: List["Period"] = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class PermanentLockedAccount(betterproto.Message):
    """
    PermanentLockedAccount implements the VestingAccount interface. It does not
    ever release coins, locking them indefinitely. Coins in this account can
    still be used for delegating and for governance votes even while locked.
    Since: cosmos-sdk 0.43
    """

    base_vesting_account: "BaseVestingAccount" = betterproto.message_field(1)


class MsgStub(betterproto.ServiceStub):
    async def create_vesting_account(
        self,
        msg_create_vesting_account: "MsgCreateVestingAccount",
        *,
        timeout: Optional[float] = None,
        deadline: Optional["Deadline"] = None,
        metadata: Optional["MetadataLike"] = None
    ) -> "MsgCreateVestingAccountResponse":
        return await self._unary_unary(
            "/cosmos.vesting.v1beta1.Msg/CreateVestingAccount",
            msg_create_vesting_account,
            MsgCreateVestingAccountResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MsgBase(ServiceBase):
    async def create_vesting_account(
        self, msg_create_vesting_account: "MsgCreateVestingAccount"
    ) -> "MsgCreateVestingAccountResponse":
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_create_vesting_account(
        self,
        stream: "grpclib.server.Stream[MsgCreateVestingAccount, MsgCreateVestingAccountResponse]",
    ) -> None:
        request = await stream.recv_message()
        response = await self.create_vesting_account(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            "/cosmos.vesting.v1beta1.Msg/CreateVestingAccount": grpclib.const.Handler(
                self.__rpc_create_vesting_account,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgCreateVestingAccount,
                MsgCreateVestingAccountResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/crypto/secp256r1/keys.proto
# plugin: python-betterproto
from dataclasses import dataclass

import betterproto


@dataclass(eq=False, repr=False)
class PubKey(betterproto.Message):
    """PubKey defines a secp256r1 ECDSA public key."""

    key: bytes = betterproto.bytes_field(1)
    """
    Point on secp256r1 curve in a compressed representation as specified in
    section 4.3.6 of ANSI X9.62:
    https://webstore.ansi.org/standards/ascx9/ansix9621998
    """


@dataclass(eq=False, repr=False)
class PrivKey(betterproto.Message):
    """PrivKey defines a secp256r1 ECDSA private key."""

    secret: bytes = betterproto.bytes_field(1)
    """secret number serialized using big-endian encoding"""
